@@ -48,8 +48,8 @@ class EXTENSIONSYSTEM_EXPORT PluginView : public QWidget
     Q_OBJECT
 
 public:
-    explicit PluginView(QWidget *parent = 0);
-    ~PluginView();
+    explicit PluginView(QWidget *parent = nullptr);
+    ~PluginView() override;
 
     PluginSpec *currentPlugin() const;
     void setFilter(const QString &filter);
