@@ -38,6 +38,7 @@
 #include <utils/qtcassert.h>
 
 #include <QApplication>
+#include <QDebug>
 #include <QDir>
 #include <QFile>
 #include <QString>
@@ -99,7 +100,7 @@ class LookupMember: public MemberProcessor
 
 public:
     LookupMember(const QString &name)
-        : m_name(name), m_value(0) {}
+        : m_name(name), m_value(nullptr) {}
 
     const Value *value() const { return m_value; }
 
@@ -290,7 +291,7 @@ void CppComponentValue::processMembers(MemberProcessor *processor) const
         signatures->reserve(m_metaObject->methodCount());
         for (int index = 0; index < m_metaObject->methodCount(); ++index)
             signatures->append(new MetaFunction(m_metaObject->method(index), valueOwner()));
-        if (!m_metaSignatures.testAndSetOrdered(0, signatures)) {
+        if (!m_metaSignatures.testAndSetOrdered(nullptr, signatures)) {
             delete signatures;
             signatures = m_metaSignatures.load();
         }
@@ -394,6 +395,8 @@ const Value *CppComponentValue::valueForCppName(const QString &typeName) const
         return valueOwner()->realValue();
     } else if (typeName == QLatin1String("QFont")) {
         return valueOwner()->qmlFontObject();
+    } else if (typeName == QLatin1String("QPalette")) {
+        return valueOwner()->qmlPaletteObject();
     } else if (typeName == QLatin1String("QPoint")
             || typeName == QLatin1String("QPointF")
             || typeName == QLatin1String("QVector2D")) {
@@ -500,7 +503,7 @@ FakeMetaEnum CppComponentValue::getEnum(const QString &typeName, const CppCompon
         }
     }
     if (foundInScope)
-        *foundInScope = 0;
+        *foundInScope = nullptr;
     return FakeMetaEnum();
 }
 
@@ -514,8 +517,8 @@ const QmlEnumValue *CppComponentValue::getEnumValue(const QString &typeName, con
         }
     }
     if (foundInScope)
-        *foundInScope = 0;
-    return 0;
+        *foundInScope = nullptr;
+    return nullptr;
 }
 
 const ObjectValue *CppComponentValue::signalScope(const QString &signalName) const
@@ -534,7 +537,7 @@ const ObjectValue *CppComponentValue::signalScope(const QString &signalName) con
             const QStringList &parameterTypes = method.parameterTypes();
             QTC_ASSERT(parameterNames.size() == parameterTypes.size(), continue);
 
-            ObjectValue *scope = valueOwner()->newObject(/*prototype=*/0);
+            ObjectValue *scope = valueOwner()->newObject(/*prototype=*/nullptr);
             for (int i = 0; i < parameterNames.size(); ++i) {
                 const QString &name = parameterNames.at(i);
                 const QString &type = parameterTypes.at(i);
@@ -544,7 +547,7 @@ const ObjectValue *CppComponentValue::signalScope(const QString &signalName) con
             }
             scopes->insert(generatedSlotName(method.methodName()), scope);
         }
-        if (!m_signalScopes.testAndSetOrdered(0, scopes)) {
+        if (!m_signalScopes.testAndSetOrdered(nullptr, scopes)) {
             delete scopes;
             scopes = m_signalScopes.load();
         }
@@ -708,137 +711,137 @@ bool Value::getSourceLocation(QString *, int *, int *) const
 
 const NullValue *Value::asNullValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const UndefinedValue *Value::asUndefinedValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const UnknownValue *Value::asUnknownValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const NumberValue *Value::asNumberValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const IntValue *Value::asIntValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const RealValue *Value::asRealValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const BooleanValue *Value::asBooleanValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const StringValue *Value::asStringValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const UrlValue *Value::asUrlValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ObjectValue *Value::asObjectValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const FunctionValue *Value::asFunctionValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const Reference *Value::asReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ColorValue *Value::asColorValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const AnchorLineValue *Value::asAnchorLineValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const CppComponentValue *Value::asCppComponentValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTObjectValue *Value::asAstObjectValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const QmlEnumValue *Value::asQmlEnumValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const QmlPrototypeReference *Value::asQmlPrototypeReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTPropertyReference *Value::asAstPropertyReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTVariableReference *Value::asAstVariableReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const Internal::QtObjectPrototypeReference *Value::asQtObjectPrototypeReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTSignal *Value::asAstSignal() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTFunctionValue *Value::asAstFunctionValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const Function *Value::asFunction() const
 {
-    return 0;
+    return nullptr;
 }
 
 const MetaFunction *Value::asMetaFunction() const
 {
-    return 0;
+    return nullptr;
 }
 
 const JSImportScope *Value::asJSImportScope() const
 {
-    return 0;
+    return nullptr;
 }
 
 const TypeScope *Value::asTypeScope() const
 {
-    return 0;
+    return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1003,7 +1006,7 @@ bool MemberProcessor::processGeneratedSlot(const QString &, const Value *)
 
 ObjectValue::ObjectValue(ValueOwner *valueOwner, const QString &originId)
     : m_valueOwner(valueOwner), m_originId(originId),
-      _prototype(0)
+      _prototype(nullptr)
 {
     valueOwner->registerValue(this);
 }
@@ -1098,11 +1101,7 @@ bool ObjectValue::checkPrototype(const ObjectValue *, QSet<const ObjectValue *>
 
 void ObjectValue::processMembers(MemberProcessor *processor) const
 {
-    QHashIterator<QString, PropertyData> it(m_members);
-
-    while (it.hasNext()) {
-        it.next();
-
+    for (auto it = m_members.cbegin(), end = m_members.cend(); it != end; ++it) {
         if (! processor->processProperty(it.key(), it.value().value, it.value().propertyInfo))
             break;
     }
@@ -1126,7 +1125,7 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
         }
     }
 
-    const ObjectValue *prototypeObject = 0;
+    const ObjectValue *prototypeObject = nullptr;
 
     if (examinePrototypes && context) {
         PrototypeIterator iter(this, context);
@@ -1139,13 +1138,13 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
     }
 
     if (foundInObject)
-        *foundInObject = 0;
+        *foundInObject = nullptr;
 
-    return 0;
+    return nullptr;
 }
 
 PrototypeIterator::PrototypeIterator(const ObjectValue *start, const Context *context)
-    : m_current(0)
+    : m_current(nullptr)
     , m_next(start)
     , m_context(context)
     , m_error(NoError)
@@ -1155,7 +1154,7 @@ PrototypeIterator::PrototypeIterator(const ObjectValue *start, const Context *co
 }
 
 PrototypeIterator::PrototypeIterator(const ObjectValue *start, const ContextPtr &context)
-    : m_current(0)
+    : m_current(nullptr)
     , m_next(start)
     , m_context(context.data())
     , m_error(NoError)
@@ -1183,7 +1182,7 @@ bool PrototypeIterator::hasNext()
     }
     if (m_prototypes.contains(m_next)) {
         m_error = CycleError;
-        m_next = 0;
+        m_next = nullptr;
         return false;
     }
     return true;
@@ -1194,17 +1193,17 @@ const ObjectValue *PrototypeIterator::next()
     if (hasNext()) {
         m_current = m_next;
         m_prototypes += m_next;
-        m_next = 0;
+        m_next = nullptr;
         return m_current;
     }
-    return 0;
+    return nullptr;
 }
 
 const ObjectValue *PrototypeIterator::peekNext()
 {
     if (hasNext())
         return m_next;
-    return 0;
+    return nullptr;
 }
 
 PrototypeIterator::Error PrototypeIterator::error() const
@@ -1273,7 +1272,7 @@ void FunctionValue::accept(ValueVisitor *visitor) const
 
 Function::Function(ValueOwner *valueOwner)
     : FunctionValue(valueOwner)
-    , m_returnValue(0)
+    , m_returnValue(nullptr)
     , m_optionalNamedArgumentCount(0)
     , m_isVariadic(false)
 {
@@ -1368,7 +1367,7 @@ CppQmlTypesLoader::BuiltinObjects CppQmlTypesLoader::loadQmlTypes(const QFileInf
             file.close();
 
 
-            parseQmlTypeDescriptions(contents, &newObjects, 0, &newDependencies, &error, &warning,
+            parseQmlTypeDescriptions(contents, &newObjects, nullptr, &newDependencies, &error, &warning,
                                      qmlTypeFile.absoluteFilePath());
         } else {
             error = file.errorString();
@@ -1421,7 +1420,7 @@ void CppQmlTypesLoader::parseQmlTypeDescriptions(const QByteArray &contents,
 }
 
 CppQmlTypes::CppQmlTypes(ValueOwner *valueOwner)
-    : m_cppContextProperties(0)
+    : m_cppContextProperties(nullptr)
     , m_valueOwner(valueOwner)
 
 {
@@ -1607,13 +1606,13 @@ const ObjectValue *CppQmlTypes::cppContextProperties() const
 
 
 ConvertToNumber::ConvertToNumber(ValueOwner *valueOwner)
-    : m_valueOwner(valueOwner), m_result(0)
+    : m_valueOwner(valueOwner), m_result(nullptr)
 {
 }
 
 const Value *ConvertToNumber::operator()(const Value *value)
 {
-    const Value *previousValue = switchResult(0);
+    const Value *previousValue = switchResult(nullptr);
 
     if (value)
         value->accept(this);
@@ -1670,13 +1669,13 @@ void ConvertToNumber::visit(const FunctionValue *object)
 }
 
 ConvertToString::ConvertToString(ValueOwner *valueOwner)
-    : m_valueOwner(valueOwner), m_result(0)
+    : m_valueOwner(valueOwner), m_result(nullptr)
 {
 }
 
 const Value *ConvertToString::operator()(const Value *value)
 {
-    const Value *previousValue = switchResult(0);
+    const Value *previousValue = switchResult(nullptr);
 
     if (value)
         value->accept(this);
@@ -1733,13 +1732,13 @@ void ConvertToString::visit(const FunctionValue *object)
 }
 
 ConvertToObject::ConvertToObject(ValueOwner *valueOwner)
-    : m_valueOwner(valueOwner), m_result(0)
+    : m_valueOwner(valueOwner), m_result(nullptr)
 {
 }
 
 const Value *ConvertToObject::operator()(const Value *value)
 {
-    const Value *previousValue = switchResult(0);
+    const Value *previousValue = switchResult(nullptr);
 
     if (value)
         value->accept(this);
@@ -1855,7 +1854,7 @@ ASTObjectValue::ASTObjectValue(UiQualifiedId *typeName,
                                const Document *doc,
                                ValueOwner *valueOwner)
     : ObjectValue(valueOwner, doc->importId()),
-      m_typeName(typeName), m_initializer(initializer), m_doc(doc), m_defaultPropertyRef(0)
+      m_typeName(typeName), m_initializer(initializer), m_doc(doc), m_defaultPropertyRef(nullptr)
 {
     if (m_initializer) {
         for (UiObjectMemberList *it = m_initializer->members; it; it = it->next) {
@@ -1996,7 +1995,7 @@ public:
     }
 
 protected:
-    bool visit(ArrayMemberExpression *ast)
+    bool visit(ArrayMemberExpression *ast) override
     {
         if (IdentifierExpression *idExp = cast<IdentifierExpression *>(ast->base)) {
             if (idExp->name == QLatin1String("arguments"))
@@ -2006,8 +2005,12 @@ protected:
     }
 
     // don't go into nested functions
-    bool visit(Program *) { return false; }
-    bool visit(StatementList *) { return false; }
+    bool visit(Program *) override { return false; }
+    bool visit(StatementList *) override { return false; }
+
+    void throwRecursionDepthError() override {
+        qWarning("Warning: Hit maximum recursion error visiting AST in UsesArgumentsArray");
+    }
 };
 } // anonymous namespace
 
@@ -2166,7 +2169,7 @@ ASTSignal::ASTSignal(UiPublicMember *ast, const Document *doc, ValueOwner *value
     const QString &signalName = ast->name.toString();
     m_slotName = generatedSlotName(signalName);
 
-    ObjectValue *v = valueOwner->newObject(/*prototype=*/0);
+    ObjectValue *v = valueOwner->newObject(/*prototype=*/nullptr);
     for (UiParameterList *it = ast->parameters; it; it = it->next) {
         if (!it->name.isEmpty())
             v->setMember(it->name.toString(), valueOwner->defaultValueForBuiltinType(it->type->name.toString()));
@@ -2222,7 +2225,7 @@ bool ASTSignal::getSourceLocation(QString *fileName, int *line, int *column) con
 
 ImportInfo::ImportInfo()
     : m_type(ImportType::Invalid)
-    , m_ast(0)
+    , m_ast(nullptr)
 {
 }
 
@@ -2332,7 +2335,7 @@ UiImport *ImportInfo::ast() const
 }
 
 Import::Import()
-    : object(0), valid(false), used(false)
+    : object(nullptr), valid(false), used(false)
 {}
 
 Import::Import(const Import &other)
@@ -2340,6 +2343,16 @@ Import::Import(const Import &other)
       valid(other.valid), used(false)
 { }
 
+Import &Import::operator=(const Import &other)
+{
+    object = other.object;
+    info = other.info;
+    libraryPath = other.libraryPath;
+    valid = other.valid;
+    used = false;
+    return *this;
+}
+
 TypeScope::TypeScope(const Imports *imports, ValueOwner *valueOwner)
     : ObjectValue(valueOwner)
     , m_imports(imports)
@@ -2349,10 +2362,15 @@ TypeScope::TypeScope(const Imports *imports, ValueOwner *valueOwner)
 const Value *TypeScope::lookupMember(const QString &name, const Context *context,
                                            const ObjectValue **foundInObject, bool) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    if (const ObjectValue *value = m_imports->resolveAliasAndMarkUsed(name)) {
+        if (foundInObject)
+            *foundInObject = this;
+        return value;
+    }
+
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2360,32 +2378,21 @@ const Value *TypeScope::lookupMember(const QString &name, const Context *context
         if (info.type() == ImportType::File || info.type() == ImportType::QrcFile)
             continue;
 
-        if (!info.as().isEmpty()) {
-            if (info.as() == name) {
-                if (foundInObject)
-                    *foundInObject = this;
-                i.used = true;
-                return import;
-            }
-            continue;
-        }
-
         if (const Value *v = import->lookupMember(name, context, foundInObject)) {
             i.used = true;
             return v;
         }
     }
     if (foundInObject)
-        *foundInObject = 0;
-    return 0;
+        *foundInObject = nullptr;
+    return nullptr;
 }
 
 void TypeScope::processMembers(MemberProcessor *processor) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2414,35 +2421,17 @@ JSImportScope::JSImportScope(const Imports *imports, ValueOwner *valueOwner)
 const Value *JSImportScope::lookupMember(const QString &name, const Context *,
                                          const ObjectValue **foundInObject, bool) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
-        const ObjectValue *import = i.object;
-        const ImportInfo &info = i.info;
-
-        // JS imports are always: import "somefile.js" as Foo
-        if (info.type() != ImportType::File && info.type() != ImportType::QrcFile)
-            continue;
-
-        if (info.as() == name) {
-            if (foundInObject)
-                *foundInObject = this;
-            i.used = true;
-            return import;
-        }
-    }
+    const ObjectValue *value = m_imports->resolveAliasAndMarkUsed(name);
     if (foundInObject)
-        *foundInObject = 0;
-    return 0;
+        *foundInObject = value ? this : nullptr;
+    return value;
 }
 
 void JSImportScope::processMembers(MemberProcessor *processor) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2462,10 +2451,31 @@ Imports::Imports(ValueOwner *valueOwner)
     , m_importFailed(false)
 {}
 
+class MemberCopy : public MemberProcessor
+{
+public:
+    explicit MemberCopy(ObjectValue *value) : m_value(value) {}
+    bool processProperty(const QString &name, const Value *value,
+                         const PropertyInfo & /*propertyInfo*/) override
+    {
+        m_value->setMember(name, value);
+        return true;
+    }
+private:
+    ObjectValue *m_value = nullptr;
+};
+
 void Imports::append(const Import &import)
 {
     // when doing lookup, imports with 'as' clause are looked at first
     if (!import.info.as().isEmpty()) {
+        const QString alias = import.info.as();
+        if (!m_aliased.contains(alias))
+            m_aliased.insert(alias, m_typeScope->valueOwner()->newObject(nullptr));
+        ObjectValue *obj = m_aliased[alias];
+        MemberCopy copyProcessor(obj);
+        import.object->processMembers(&copyProcessor);
+
         m_imports.append(import);
     } else {
         // find first as-import and prepend
@@ -2495,10 +2505,8 @@ ImportInfo Imports::info(const QString &name, const Context *context) const
     if (dotIdx != -1)
         firstId = firstId.left(dotIdx);
 
-    QListIterator<Import> it(m_imports);
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    for (int pos = m_imports.size(); --pos >= 0; ) {
+        const Import &i = m_imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2521,10 +2529,8 @@ ImportInfo Imports::info(const QString &name, const Context *context) const
 
 QString Imports::nameForImportedObject(const ObjectValue *value, const Context *context) const
 {
-    QListIterator<Import> it(m_imports);
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    for (int pos = m_imports.size(); --pos >= 0; ) {
+        const Import &i = m_imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2556,6 +2562,11 @@ const QList<Import> &Imports::all() const
     return m_imports;
 }
 
+const ObjectValue *Imports::aliased(const QString &name) const
+{
+    return m_aliased.value(name, nullptr);
+}
+
 const TypeScope *Imports::typeScope() const
 {
     return m_typeScope;
@@ -2566,6 +2577,20 @@ const JSImportScope *Imports::jsImportScope() const
     return m_jsImportScope;
 }
 
+const ObjectValue *Imports::resolveAliasAndMarkUsed(const QString &name) const
+{
+    if (const ObjectValue *value = m_aliased.value(name, nullptr)) {
+        // mark all respective ImportInfo objects to avoid dropping imports (QmlDesigner) on rewrite
+        for (const Import &i : qAsConst(m_imports)) {
+            const ImportInfo &info = i.info;
+            if (info.as() == name)
+                i.used = true; // FIXME: This evilly modifies a 'const' object
+        }
+        return value;
+    }
+    return nullptr;
+}
+
 #ifdef QT_DEBUG
 
 class MemberDumper: public MemberProcessor
@@ -2607,10 +2632,8 @@ public:
 void Imports::dump() const
 {
     qCDebug(qmljsLog) << "Imports contents, in search order:";
-    QListIterator<Import> it(m_imports);
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    for (int pos = m_imports.size(); --pos >= 0; ) {
+        const Import &i = m_imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
