@@ -24,11 +24,19 @@
 ****************************************************************************/
 
 #include "bineditorwidget.h"
+#include "bineditorservice.h"
+#include "markup.h"
 
+#include <coreplugin/coreconstants.h>
+#include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/editormanager/ieditor.h>
+
+#include <texteditor/behaviorsettings.h>
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditorconstants.h>
 #include <texteditor/texteditorsettings.h>
-#include <coreplugin/editormanager/ieditor.h>
+
+#include <utils/fadingindicator.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
 
@@ -51,6 +59,8 @@
 #include <QToolTip>
 #include <QWheelEvent>
 
+using namespace Core;
+
 static QByteArray calculateHexPattern(const QByteArray &pattern)
 {
     QByteArray result;
@@ -69,12 +79,68 @@ static QByteArray calculateHexPattern(const QByteArray &pattern)
 }
 
 namespace BinEditor {
+namespace Internal {
+
+class BinEditorWidgetPrivate : public EditorService
+{
+public:
+    BinEditorWidgetPrivate(BinEditorWidget *widget) : q(widget) {}
+    ~BinEditorWidgetPrivate() override { if (m_aboutToBeDestroyedHandler) m_aboutToBeDestroyedHandler(); }
+
+    QWidget *widget() override { return q; }
+    IEditor *editor() override { return q->editor(); }
+
+    void setReadOnly(bool on) override { q->setReadOnly(on); }
+    void setNewWindowRequestAllowed(bool on) override { q->setNewWindowRequestAllowed(on); }
+
+    void setFinished() override
+    {
+        q->setReadOnly(true);
+        m_fetchDataHandler = {};
+        m_newWindowRequestHandler = {};
+        m_newRangeRequestHandler = {};
+        m_dataChangedHandler = {};
+        m_watchPointRequestHandler = {};
+    }
+
+    void setSizes(quint64 address, qint64 range, int blockSize) override { q->setSizes(address, range, blockSize); }
+    void setCursorPosition(qint64 pos) override { q->setCursorPosition(pos); }
+    void updateContents() override { q->updateContents(); }
+    void addData(quint64 address, const QByteArray &data) override { q->addData(address, data); }
+
+    void clearMarkup() override { m_markup.clear(); }
+    void addMarkup(quint64 a, quint64 l, const QColor &c, const QString &t) override { m_markup.append(Markup(a, l, c, t)); }
+    void commitMarkup() override { q->setMarkup(m_markup); }
+
+    void setFetchDataHandler(const std::function<void(quint64)> &cb) override { m_fetchDataHandler = cb; }
+    void setNewWindowRequestHandler(const std::function<void(quint64)> &cb) override { m_newWindowRequestHandler = cb; }
+    void setNewRangeRequestHandler(const std::function<void(quint64)> &cb) override { m_newRangeRequestHandler = cb; }
+    void setDataChangedHandler(const std::function<void(quint64, const QByteArray &)> &cb) override { m_dataChangedHandler = cb; }
+    void setWatchPointRequestHandler(const std::function<void(quint64, uint)> &cb) override { m_watchPointRequestHandler = cb; }
+    void setAboutToBeDestroyedHandler(const std::function<void()> & cb) override { m_aboutToBeDestroyedHandler = cb; }
+
+    void fetchData(quint64 address) { if (m_fetchDataHandler) m_fetchDataHandler(address); }
+    void requestNewWindow(quint64 address) { if (m_newWindowRequestHandler) m_newWindowRequestHandler(address); }
+    void requestWatchPoint(quint64 address, int size) { if (m_watchPointRequestHandler) m_watchPointRequestHandler(address, size); }
+    void requestNewRange(quint64 address) { if (m_newRangeRequestHandler) m_newRangeRequestHandler(address); }
+    void announceChangedData(quint64 address, const QByteArray &ba) { if (m_dataChangedHandler) m_dataChangedHandler(address, ba); }
+
+private:
+    BinEditorWidget *q;
+    std::function<void(quint64)> m_fetchDataHandler;
+    std::function<void(quint64)> m_newWindowRequestHandler;
+    std::function<void(quint64)> m_newRangeRequestHandler;
+    std::function<void(quint64, const QByteArray &)> m_dataChangedHandler;
+    std::function<void(quint64, uint)> m_watchPointRequestHandler;
+    std::function<void()> m_aboutToBeDestroyedHandler;
+    QList<Markup> m_markup;
+};
 
 BinEditorWidget::BinEditorWidget(QWidget *parent)
-    : QAbstractScrollArea(parent)
+    : QAbstractScrollArea(parent), d(new BinEditorWidgetPrivate(this))
 {
     m_bytesPerLine = 16;
-    m_ieditor = 0;
+    m_ieditor = nullptr;
     m_baseAddr = 0;
     m_blockSize = 4096;
     m_size = 0;
@@ -102,6 +168,12 @@ BinEditorWidget::BinEditorWidget(QWidget *parent)
 
 BinEditorWidget::~BinEditorWidget()
 {
+    delete d;
+}
+
+EditorService *BinEditorWidget::editorService() const
+{
+    return d;
 }
 
 void BinEditorWidget::init()
@@ -113,35 +185,35 @@ void BinEditorWidget::init()
     m_descent = fm.descent();
     m_ascent = fm.ascent();
     m_lineHeight = fm.lineSpacing();
-    m_charWidth = fm.width(QChar(QLatin1Char('M')));
+    m_charWidth = fm.horizontalAdvance(QChar(QLatin1Char('M')));
     m_margin = m_charWidth;
-    m_columnWidth = 2 * m_charWidth + fm.width(QChar(QLatin1Char(' ')));
+    m_columnWidth = 2 * m_charWidth + fm.horizontalAdvance(QChar(QLatin1Char(' ')));
     m_numLines = m_size / m_bytesPerLine + 1;
     m_numVisibleLines = viewport()->height() / m_lineHeight;
     m_textWidth = m_bytesPerLine * m_charWidth + m_charWidth;
-    int numberWidth = fm.width(QChar(QLatin1Char('9')));
+    int numberWidth = fm.horizontalAdvance(QChar(QLatin1Char('9')));
     m_labelWidth = 2*m_addressBytes * numberWidth + (m_addressBytes - 1)/2 * m_charWidth;
 
     int expectedCharWidth = m_columnWidth / 3;
     const char *hex = "0123456789abcdef";
     m_isMonospacedFont = true;
     while (*hex) {
-        if (fm.width(QLatin1Char(*hex)) != expectedCharWidth) {
+        if (fm.horizontalAdvance(QLatin1Char(*hex)) != expectedCharWidth) {
             m_isMonospacedFont = false;
             break;
         }
         ++hex;
     }
 
-    if (m_isMonospacedFont && fm.width(QLatin1String("M M ")) != m_charWidth * 4) {
+    if (m_isMonospacedFont && fm.horizontalAdvance(QLatin1String("M M ")) != m_charWidth * 4) {
         // On Qt/Mac, monospace font widths may have a fractional component
         // This breaks the assumption that width("MMM") == width('M') * 3
 
         m_isMonospacedFont = false;
-        m_columnWidth = fm.width(QLatin1String("MMM"));
+        m_columnWidth = fm.horizontalAdvance(QLatin1String("MMM"));
         m_labelWidth = m_addressBytes == 4
-            ? fm.width(QLatin1String("MMMM:MMMM"))
-            : fm.width(QLatin1String("MMMM:MMMM:MMMM:MMMM"));
+            ? fm.horizontalAdvance(QLatin1String("MMMM:MMMM"))
+            : fm.horizontalAdvance(QLatin1String("MMMM:MMMM:MMMM:MMMM"));
     }
 
     horizontalScrollBar()->setRange(0, 2 * m_margin + m_bytesPerLine * m_columnWidth
@@ -153,10 +225,9 @@ void BinEditorWidget::init()
 }
 
 
-void BinEditorWidget::addData(quint64 block, const QByteArray &data)
+void BinEditorWidget::addData(quint64 addr, const QByteArray &data)
 {
     QTC_ASSERT(data.size() == m_blockSize, return);
-    const quint64 addr = block * m_blockSize;
     if (addr >= m_baseAddr && addr <= m_baseAddr + m_size - 1) {
         if (m_data.size() * m_blockSize >= 64 * 1024 * 1024)
             m_data.clear();
@@ -176,13 +247,11 @@ bool BinEditorWidget::requestDataAt(qint64 pos) const
     it = m_data.find(block);
     if (it != m_data.end())
         return true;
-    if (!m_requests.contains(block)) {
-        m_requests.insert(block);
-        emit const_cast<BinEditorWidget*>(this)->
-            dataRequested(m_baseAddr / m_blockSize + block);
-        return true;
-    }
-    return false;
+    if (m_requests.contains(block))
+        return false;
+    m_requests.insert(block);
+    d->fetchData((m_baseAddr / m_blockSize + block) * m_blockSize);
+    return true;
 }
 
 bool BinEditorWidget::requestOldDataAt(qint64 pos) const
@@ -215,7 +284,7 @@ void BinEditorWidget::changeDataAt(qint64 pos, char c)
         }
     }
 
-    emit dataChanged(m_baseAddr + pos, QByteArray(1, c));
+    d->announceChangedData(m_baseAddr + pos, QByteArray(1, c));
 }
 
 QByteArray BinEditorWidget::dataMid(qint64 from, int length, bool old) const
@@ -429,9 +498,9 @@ void BinEditorWidget::scrollContentsBy(int dx, int dy)
     const QScrollBar * const scrollBar = verticalScrollBar();
     const int scrollPos = scrollBar->value();
     if (dy <= 0 && scrollPos == scrollBar->maximum())
-        emit newRangeRequested(baseAddress() + m_size);
+        d->requestNewRange(baseAddress() + m_size);
     else if (dy >= 0 && scrollPos == scrollBar->minimum())
-        emit newRangeRequested(baseAddress());
+        d->requestNewRange(baseAddress());
 }
 
 void BinEditorWidget::changeEvent(QEvent *e)
@@ -449,11 +518,16 @@ void BinEditorWidget::changeEvent(QEvent *e)
 void BinEditorWidget::wheelEvent(QWheelEvent *e)
 {
     if (e->modifiers() & Qt::ControlModifier) {
-        const int delta = e->delta();
-        if (delta < 0)
-            zoomOut();
-        else if (delta > 0)
-            zoomIn();
+        if (!TextEditor::TextEditorSettings::behaviorSettings().m_scrollWheelZooming) {
+            // When the setting is disabled globally,
+            // we have to skip calling QAbstractScrollArea::wheelEvent()
+            // that changes zoom in it.
+            return;
+        }
+
+        const float delta = e->angleDelta().y() / 120.f;
+        if (delta != 0)
+            zoomF(delta);
         return;
     }
     QAbstractScrollArea::wheelEvent(e);
@@ -474,31 +548,38 @@ QRect BinEditorWidget::cursorRect() const
     return QRect(x, y, w, m_lineHeight);
 }
 
-int BinEditorWidget::posAt(const QPoint &pos) const
+Utils::optional<qint64> BinEditorWidget::posAt(const QPoint &pos, bool includeEmptyArea) const
 {
-    int xoffset = horizontalScrollBar()->value();
+    const int xoffset = horizontalScrollBar()->value();
     int x = xoffset + pos.x() - m_margin - m_labelWidth;
+    if (!includeEmptyArea && x < 0)
+        return Utils::nullopt;
     int column = qMin(15, qMax(0,x) / m_columnWidth);
-    qint64 topLine = verticalScrollBar()->value();
-    qint64 line = pos.y() / m_lineHeight;
-
+    const qint64 topLine = verticalScrollBar()->value();
+    const qint64 line = topLine + pos.y() / m_lineHeight;
 
+    // "clear text" area
     if (x > m_bytesPerLine * m_columnWidth + m_charWidth/2) {
         x -= m_bytesPerLine * m_columnWidth + m_charWidth;
-        for (column = 0; column < 15; ++column) {
-            int dataPos = (topLine + line) * m_bytesPerLine + column;
+        for (column = 0; column < 16; ++column) {
+            const qint64 dataPos = line * m_bytesPerLine + column;
             if (dataPos < 0 || dataPos >= m_size)
                 break;
             QChar qc(QLatin1Char(dataAt(dataPos)));
             if (!qc.isPrint())
                 qc = 0xB7;
-            x -= fontMetrics().width(qc);
+            x -= fontMetrics().horizontalAdvance(qc);
             if (x <= 0)
                 break;
         }
+        if (!includeEmptyArea && x > 0) // right of the text area
+            return Utils::nullopt;
     }
 
-    return qMin(m_size, qMin(m_numLines, topLine + line) * m_bytesPerLine) + column;
+    const qint64 bytePos = line * m_bytesPerLine + column;
+    if (!includeEmptyArea && bytePos >= m_size)
+        return Utils::nullopt;
+    return qMin(m_size - 1, bytePos);
 }
 
 bool BinEditorWidget::inTextArea(const QPoint &pos) const
@@ -532,7 +613,6 @@ int BinEditorWidget::dataIndexOf(const QByteArray &pattern, qint64 from, bool ca
 
     QByteArray buffer;
     buffer.resize(m_blockSize + trailing);
-    char *b = buffer.data();
     QByteArrayMatcher matcher(pattern);
 
     qint64 block = from / m_blockSize;
@@ -541,6 +621,7 @@ int BinEditorWidget::dataIndexOf(const QByteArray &pattern, qint64 from, bool ca
         if (!requestDataAt(block * m_blockSize))
             return -1;
         QByteArray data = blockData(block);
+        char *b = buffer.data();
         ::memcpy(b, b + m_blockSize, trailing);
         ::memcpy(b + trailing, data.constData(), m_blockSize);
 
@@ -564,16 +645,16 @@ int BinEditorWidget::dataLastIndexOf(const QByteArray &pattern, qint64 from, boo
 
     QByteArray buffer;
     buffer.resize(m_blockSize + trailing);
-    char *b = buffer.data();
 
     if (from == -1)
         from = m_size;
     int block = from / m_blockSize;
     const int lowerBound = qMax(qint64(0), from - SearchStride);
     while (from > lowerBound) {
-        if (!requestDataAt(block * m_blockSize))
+        if (!requestDataAt(qint64(block) * m_blockSize))
             return -1;
         QByteArray data = blockData(block);
+        char *b = buffer.data();
         ::memcpy(b + m_blockSize, b, trailing);
         ::memcpy(b, data.constData(), m_blockSize);
 
@@ -584,7 +665,7 @@ int BinEditorWidget::dataLastIndexOf(const QByteArray &pattern, qint64 from, boo
         if (pos >= 0)
             return pos + block * m_blockSize;
         --block;
-        from = block * m_blockSize + (m_blockSize-1) + trailing;
+        from = qint64(block) * m_blockSize + (m_blockSize-1) + trailing;
     }
     return lowerBound == 0 ? -1 : -2;
 }
@@ -743,7 +824,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
 
     QString itemString(m_bytesPerLine*3, QLatin1Char(' '));
     QChar *itemStringData = itemString.data();
-    char changedString[160] = { false };
+    char changedString[160] = {false};
     QTC_ASSERT((size_t)m_bytesPerLine < sizeof(changedString), return);
     const char *hex = "0123456789abcdef";
 
@@ -832,18 +913,18 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
                 if (color.isValid()) {
                     painter.fillRect(item_x - m_charWidth/2, y-m_ascent, m_columnWidth, m_lineHeight, color);
                     int printable_item_x = -xoffset + m_margin + m_labelWidth + m_bytesPerLine * m_columnWidth + m_charWidth
-                                           + fm.width(printable.left(c));
+                                           + fm.horizontalAdvance(printable.left(c));
                     painter.fillRect(printable_item_x, y-m_ascent,
-                                     fm.width(printable.at(c)),
+                                     fm.horizontalAdvance(printable.at(c)),
                                      m_lineHeight, color);
                 }
 
                 if (!isFullySelected && pos >= selStart && pos <= selEnd) {
                     selectionRect |= QRect(item_x - m_charWidth/2, y-m_ascent, m_columnWidth, m_lineHeight);
                     int printable_item_x = -xoffset + m_margin + m_labelWidth + m_bytesPerLine * m_columnWidth + m_charWidth
-                                           + fm.width(printable.left(c));
+                                           + fm.horizontalAdvance(printable.left(c));
                     printableSelectionRect |= QRect(printable_item_x, y-m_ascent,
-                                                    fm.width(printable.at(c)),
+                                                    fm.horizontalAdvance(printable.at(c)),
                                                     m_lineHeight);
                 }
             }
@@ -877,7 +958,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
             paintCursorBorder(&painter, cursorRect);
             if (m_hexCursor && m_cursorVisible) {
                 if (m_lowNibble)
-                    cursorRect.adjust(fm.width(itemString.left(1)), 0, 0, 0);
+                    cursorRect.adjust(fm.horizontalAdvance(itemString.left(1)), 0, 0, 0);
                 painter.fillRect(cursorRect, Qt::red);
                 painter.save();
                 painter.setClipRect(cursorRect);
@@ -891,7 +972,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
 
         if (isFullySelected) {
                 painter.save();
-                painter.fillRect(text_x, y-m_ascent, fm.width(printable), m_lineHeight,
+                painter.fillRect(text_x, y-m_ascent, fm.horizontalAdvance(printable), m_lineHeight,
                                  palette().highlight());
                 painter.setPen(palette().highlightedText().color());
                 painter.drawText(text_x, y, printable);
@@ -909,9 +990,9 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
         }
 
         if (cursor >= 0 && !printable.isEmpty()) {
-            QRect cursorRect(text_x + fm.width(printable.left(cursor)),
+            QRect cursorRect(text_x + fm.horizontalAdvance(printable.left(cursor)),
                              y-m_ascent,
-                             fm.width(printable.at(cursor)),
+                             fm.horizontalAdvance(printable.at(cursor)),
                              m_lineHeight);
             if (m_hexCursor || !m_cursorVisible) {
                 paintCursorBorder(&painter, cursorRect);
@@ -968,7 +1049,7 @@ void BinEditorWidget::mousePressEvent(QMouseEvent *e)
     if (e->button() != Qt::LeftButton)
         return;
     MoveMode moveMode = e->modifiers() & Qt::ShiftModifier ? KeepAnchor : MoveAnchor;
-    setCursorPosition(posAt(e->pos()), moveMode);
+    setCursorPosition(posAt(e->pos()).value(), moveMode);
     setBlinkingCursorEnabled(true);
     if (m_hexCursor == inTextArea(e->pos())) {
         m_hexCursor = !m_hexCursor;
@@ -980,7 +1061,7 @@ void BinEditorWidget::mouseMoveEvent(QMouseEvent *e)
 {
     if (!(e->buttons() & Qt::LeftButton))
         return;
-    setCursorPosition(posAt(e->pos()), KeepAnchor);
+    setCursorPosition(posAt(e->pos()).value(), KeepAnchor);
     if (m_hexCursor == inTextArea(e->pos())) {
         m_hexCursor = !m_hexCursor;
         updateLines();
@@ -1044,7 +1125,7 @@ bool BinEditorWidget::event(QEvent *e)
             const QScrollBar * const scrollBar = verticalScrollBar();
             const int maximum = scrollBar->maximum();
             if (maximum && scrollBar->value() >= maximum - 1) {
-                emit newRangeRequested(baseAddress() + m_size);
+                d->requestNewRange(baseAddress() + m_size);
                 return true;
             }
             break;
@@ -1071,18 +1152,35 @@ bool BinEditorWidget::event(QEvent *e)
 
 QString BinEditorWidget::toolTip(const QHelpEvent *helpEvent) const
 {
-    int selStart = selectionStart();
-    int selEnd = selectionEnd();
-    int byteCount = selEnd - selStart + 1;
-    if (m_hexCursor == 0 || byteCount > 8)
-        return QString();
+    qint64 selStart = selectionStart();
+    qint64 selEnd = selectionEnd();
+    qint64 byteCount = std::min(8LL, selEnd - selStart + 1);
 
-    const QPoint &startPoint = offsetToPos(selStart);
-    const QPoint &endPoint = offsetToPos(selEnd + 1);
-    QRect selRect(startPoint, endPoint);
-    selRect.setHeight(m_lineHeight);
-    if (!selRect.contains(helpEvent->pos()))
-        return QString();
+    // check even position against selection line by line
+    bool insideSelection = false;
+    qint64 startInLine = selStart;
+    do {
+        const qint64 lineIndex = startInLine / m_bytesPerLine;
+        const qint64 endOfLine = (lineIndex + 1) * m_bytesPerLine - 1;
+        const qint64 endInLine = std::min(selEnd, endOfLine);
+        const QPoint &startPoint = offsetToPos(startInLine);
+        const QPoint &endPoint = offsetToPos(endInLine) + QPoint(m_columnWidth, 0);
+        QRect selectionLineRect(startPoint, endPoint);
+        selectionLineRect.setHeight(m_lineHeight);
+        if (selectionLineRect.contains(helpEvent->pos())) {
+            insideSelection = true;
+            break;
+        }
+        startInLine = endInLine + 1;
+    } while (startInLine <= selEnd);
+    if (!insideSelection) {
+        // show popup for byte under cursor
+        Utils::optional<qint64> pos = posAt(helpEvent->pos(), /*includeEmptyArea*/false);
+        if (!pos)
+            return QString();
+        selStart = pos.value();
+        byteCount = 1;
+    }
 
     quint64 bigEndianValue, littleEndianValue;
     quint64 bigEndianValueOld, littleEndianValueOld;
@@ -1352,19 +1450,25 @@ void BinEditorWidget::keyPressEvent(QKeyEvent *e)
     e->accept();
 }
 
-void BinEditorWidget::zoomIn(int range)
+static void showZoomIndicator(QWidget *editor, const int newZoom)
 {
-    QFont f = font();
-    const int newSize = f.pointSize() + range;
-    if (newSize <= 0)
-        return;
-    f.setPointSize(newSize);
-    setFont(f);
+    Utils::FadingIndicator::showText(editor,
+                                     QCoreApplication::translate("BinEditorWidget::TextEditorWidget",
+                                                                 "Zoom: %1%").arg(newZoom),
+                                     Utils::FadingIndicator::SmallText);
 }
 
-void BinEditorWidget::zoomOut(int range)
+void BinEditorWidget::zoomF(float delta)
 {
-    zoomIn(-range);
+    float step = 10.f * delta;
+    // Ensure we always zoom a minimal step in-case the resolution is more than 16x
+    if (step > 0 && step < 1)
+        step = 1;
+    else if (step < 0 && step > -1)
+        step = -1;
+
+    const int newZoom = TextEditor::TextEditorSettings::instance()->increaseFontZoom(int(step));
+    showZoomIndicator(this, newZoom);
 }
 
 void BinEditorWidget::copy(bool raw)
@@ -1452,7 +1556,7 @@ void BinEditorWidget::undo()
     setCursorPosition(cmd.position);
     if (emitModificationChanged)
         emit modificationChanged(m_undoStack.size() != m_unmodifiedState);
-    if (!m_undoStack.size())
+    if (m_undoStack.isEmpty())
         emit undoAvailable(false);
     if (m_redoStack.size() == 1)
         emit redoAvailable(true);
@@ -1473,7 +1577,7 @@ void BinEditorWidget::redo()
         emit modificationChanged(m_undoStack.size() != m_unmodifiedState);
     if (m_undoStack.size() == 1)
         emit undoAvailable(true);
-    if (!m_redoStack.size())
+    if (m_redoStack.isEmpty())
         emit redoAvailable(false);
 }
 
@@ -1486,6 +1590,8 @@ void BinEditorWidget::contextMenuEvent(QContextMenuEvent *event)
 
     auto copyAsciiAction = new QAction(tr("Copy Selection as ASCII Characters"), contextMenu);
     auto copyHexAction = new QAction(tr("Copy Selection as Hex Values"), contextMenu);
+    auto copyBeValue = new QAction(contextMenu);
+    auto copyLeValue = new QAction(contextMenu);
     auto jumpToBeAddressHereAction = new QAction(contextMenu);
     auto jumpToBeAddressNewWindowAction = new QAction(contextMenu);
     auto jumpToLeAddressHereAction = new QAction(contextMenu);
@@ -1501,10 +1607,16 @@ void BinEditorWidget::contextMenuEvent(QContextMenuEvent *event)
     quint64 leAddress = 0;
     if (byteCount <= 8) {
         asIntegers(selStart, byteCount, beAddress, leAddress);
+        copyBeValue->setText(tr("Copy 0x%1").arg(QString::number(beAddress, 16)));
+        contextMenu->addAction(copyBeValue);
+        // If the menu entries would be identical, show only one of them.
+        if (beAddress != leAddress) {
+            copyLeValue->setText(tr("Copy 0x%1").arg(QString::number(leAddress, 16)));
+            contextMenu->addAction(copyLeValue);
+        }
         setupJumpToMenuAction(contextMenu, jumpToBeAddressHereAction,
                               jumpToBeAddressNewWindowAction, beAddress);
 
-        // If the menu entries would be identical, show only one of them.
         if (beAddress != leAddress) {
             setupJumpToMenuAction(contextMenu, jumpToLeAddressHereAction,
                                   jumpToLeAddressNewWindowAction, leAddress);
@@ -1512,8 +1624,11 @@ void BinEditorWidget::contextMenuEvent(QContextMenuEvent *event)
     } else {
         jumpToBeAddressHereAction->setText(tr("Jump to Address in This Window"));
         jumpToBeAddressNewWindowAction->setText(tr("Jump to Address in New Window"));
+        copyBeValue->setText(tr("Copy Value"));
         jumpToBeAddressHereAction->setEnabled(false);
         jumpToBeAddressNewWindowAction->setEnabled(false);
+        copyBeValue->setEnabled(false);
+        contextMenu->addAction(copyBeValue);
         contextMenu->addAction(jumpToBeAddressHereAction);
         contextMenu->addAction(jumpToBeAddressNewWindowAction);
     }
@@ -1526,16 +1641,20 @@ void BinEditorWidget::contextMenuEvent(QContextMenuEvent *event)
         copy(true);
     else if (action == copyHexAction)
         copy(false);
+    else if (action == copyBeValue)
+        QApplication::clipboard()->setText("0x" + QString::number(beAddress, 16));
+    else if (action == copyLeValue)
+        QApplication::clipboard()->setText("0x" + QString::number(leAddress, 16));
     else if (action == jumpToBeAddressHereAction)
         jumpToAddress(beAddress);
     else if (action == jumpToLeAddressHereAction)
         jumpToAddress(leAddress);
     else if (action == jumpToBeAddressNewWindowAction)
-        emit newWindowRequested(beAddress);
+        d->requestNewWindow(beAddress);
     else if (action == jumpToLeAddressNewWindowAction)
-        emit newWindowRequested(leAddress);
+        d->requestNewWindow(leAddress);
     else if (action == addWatchpointAction)
-        emit addWatchpointRequested(m_baseAddr + selStart, byteCount);
+        d->requestWatchPoint(m_baseAddr + selStart, byteCount);
     delete contextMenu;
 }
 
@@ -1557,7 +1676,7 @@ void BinEditorWidget::jumpToAddress(quint64 address)
     if (address >= m_baseAddr && address < m_baseAddr + m_size)
         setCursorPosition(address - m_baseAddr);
     else
-        emit newRangeRequested(address);
+        d->requestNewRange(address);
 }
 
 void BinEditorWidget::setNewWindowRequestAllowed(bool c)
@@ -1568,7 +1687,11 @@ void BinEditorWidget::setNewWindowRequestAllowed(bool c)
 void BinEditorWidget::updateContents()
 {
     m_oldData = m_data;
-    setSizes(baseAddress() + cursorPosition(), m_size, m_blockSize);
+    m_data.clear();
+    m_modifiedData.clear();
+    m_requests.clear();
+    for (auto it = m_oldData.constBegin(), et = m_oldData.constEnd(); it != et; ++it)
+        d->fetchData(m_baseAddr + it.key());
 }
 
 QPoint BinEditorWidget::offsetToPos(qint64 offset) const
@@ -1614,4 +1737,5 @@ void BinEditorWidget::setMarkup(const QList<Markup> &markup)
     viewport()->update();
 }
 
+} // namespace Internal
 } // namespace BinEditor
