@@ -30,22 +30,19 @@
 namespace CMakeProjectManager {
 namespace Constants {
 
-const char PROJECTCONTEXT[] = "CMakeProject.ProjectContext";
 const char CMAKEMIMETYPE[]  = "text/x-cmake";
 const char CMAKEPROJECTMIMETYPE[]  = "text/x-cmake-project";
 const char CMAKE_EDITOR_ID[] = "CMakeProject.CMakeEditor";
-const char CMAKE_EDITOR_DISPLAY_NAME[] = QT_TRANSLATE_NOOP("CMakeProjectManager::Internal::CMakeEditorFactory", "CMake Editor");
 const char RUNCMAKE[] = "CMakeProject.RunCMake";
 const char CLEARCMAKECACHE[] = "CMakeProject.ClearCache";
 const char RESCANPROJECT[] = "CMakeProject.RescanProject";
 const char RUNCMAKECONTEXTMENU[] = "CMakeProject.RunCMakeContextMenu";
+const char BUILDFILECONTEXTMENU[] = "CMakeProject.BuildFileContextMenu";
+const char BUILDFILE[] = "CMakeProject.BuildFile";
 
 // Project
 const char CMAKEPROJECT_ID[] = "CMakeProjectManager.CMakeProject";
 
-// Buildconfiguration
-const char CMAKE_BC_ID[] = "CMakeProjectManager.CMakeBuildConfiguration";
-
 // Menu
 const char M_CONTEXT[] = "CMakeEditor.ContextMenu";
 
@@ -64,5 +61,8 @@ const char BUILD_TARGET_CONTEXTMENU[] = "CMake.BuildTargetContextMenu";
 // Build Step
 const char CMAKE_BUILD_STEP_ID[] = "CMakeProjectManager.MakeStep";
 
+// Features
+const char CMAKE_FEATURE_ID[] = "CMakeProjectManager.Wizard.FeatureCMake";
+
 } // namespace Constants
 } // namespace CMakeProjectManager
