@@ -56,7 +56,7 @@ static int translateModifiers(Qt::KeyboardModifiers state,
     int result = 0;
     // The shift modifier only counts when it is not used to type a symbol
     // that is only reachable using the shift key anyway
-    if ((state & Qt::ShiftModifier) && (text.size() == 0
+    if ((state & Qt::ShiftModifier) && (text.isEmpty()
                                         || !text.at(0).isPrint()
                                         || text.at(0).isLetterOrNumber()
                                         || text.at(0).isSpace()))
@@ -105,17 +105,29 @@ static bool keySequenceIsValid(const QKeySequence &sequence)
     return true;
 }
 
+static bool isTextKeySequence(const QKeySequence &sequence)
+{
+    if (sequence.isEmpty())
+        return false;
+    int key = sequence[0];
+    key &= ~(Qt::ShiftModifier | Qt::KeypadModifier);
+    if (key < Qt::Key_Escape)
+        return true;
+    return false;
+}
+
 namespace Core {
 namespace Internal {
 
 ShortcutButton::ShortcutButton(QWidget *parent)
     : QPushButton(parent)
-    , m_key({{ 0, 0, 0, 0 }})
+    , m_key({{0, 0, 0, 0}})
 {
-    setToolTip(tr("Click and type the new key sequence."));
+    // Using ShortcutButton::tr() as workaround for QTBUG-34128
+    setToolTip(ShortcutButton::tr("Click and type the new key sequence."));
     setCheckable(true);
-    m_checkedText = tr("Stop Recording");
-    m_uncheckedText = tr("Record");
+    m_checkedText = ShortcutButton::tr("Stop Recording");
+    m_uncheckedText = ShortcutButton::tr("Record");
     updateText();
     connect(this, &ShortcutButton::toggled, this, &ShortcutButton::handleToggleChange);
 }
@@ -124,7 +136,7 @@ QSize ShortcutButton::sizeHint() const
 {
     if (m_preferredWidth < 0) { // initialize size hint
         const QString originalText = text();
-        ShortcutButton *that = const_cast<ShortcutButton *>(this);
+        auto that = const_cast<ShortcutButton *>(this);
         that->setText(m_checkedText);
         m_preferredWidth = QPushButton::sizeHint().width();
         that->setText(m_uncheckedText);
@@ -152,7 +164,7 @@ bool ShortcutButton::eventFilter(QObject *obj, QEvent *evt)
         return true;
     }
     if (evt->type() == QEvent::KeyPress) {
-        QKeyEvent *k = static_cast<QKeyEvent*>(evt);
+        auto k = static_cast<QKeyEvent*>(evt);
         int nextKey = k->key();
         if (m_keyNum > 3
                 || nextKey == Qt::Key_Control
@@ -199,16 +211,46 @@ void ShortcutButton::handleToggleChange(bool toogleState)
     updateText();
     m_keyNum = m_key[0] = m_key[1] = m_key[2] = m_key[3] = 0;
     if (toogleState) {
-        if (qApp->focusWidget())
-            qApp->focusWidget()->clearFocus(); // funny things happen otherwise
+        if (QApplication::focusWidget())
+            QApplication::focusWidget()->clearFocus(); // funny things happen otherwise
         qApp->installEventFilter(this);
     } else {
         qApp->removeEventFilter(this);
     }
 }
 
-ShortcutSettingsWidget::ShortcutSettingsWidget(QWidget *parent)
-    : CommandMappings(parent)
+class ShortcutSettingsWidget final : public CommandMappings
+{
+    Q_DECLARE_TR_FUNCTIONS(Core::Internal::ShortcutSettings)
+
+public:
+    ShortcutSettingsWidget();
+    ~ShortcutSettingsWidget() final;
+
+    void apply();
+
+private:
+    void importAction() final;
+    void exportAction() final;
+    void defaultAction() final;
+    bool filterColumn(const QString &filterString, QTreeWidgetItem *item, int column) const final;
+
+    void initialize();
+    void handleCurrentCommandChanged(QTreeWidgetItem *current);
+    void resetToDefault();
+    bool validateShortcutEdit() const;
+    bool markCollisions(ShortcutItem *);
+    void setKeySequence(const QKeySequence &key);
+    void showConflicts();
+    void clear();
+
+    QList<ShortcutItem *> m_scitems;
+    QGroupBox *m_shortcutBox;
+    Utils::FancyLineEdit *m_shortcutEdit;
+    QLabel *m_warningLabel;
+};
+
+ShortcutSettingsWidget::ShortcutSettingsWidget()
 {
     setPageTitle(tr("Keyboard Shortcuts"));
     setTargetHeader(tr("Shortcut"));
@@ -282,14 +324,11 @@ ShortcutSettingsWidget::~ShortcutSettingsWidget()
     qDeleteAll(m_scitems);
 }
 
-ShortcutSettings::ShortcutSettings(QObject *parent)
-    : IOptionsPage(parent)
+ShortcutSettings::ShortcutSettings()
 {
     setId(Constants::SETTINGS_ID_SHORTCUTS);
-    setDisplayName(tr("Keyboard"));
+    setDisplayName(ShortcutSettingsWidget::tr("Keyboard"));
     setCategory(Constants::SETTINGS_CATEGORY_CORE);
-    setDisplayCategory(QCoreApplication::translate("Core", Constants::SETTINGS_TR_CATEGORY_CORE));
-    setCategoryIcon(QLatin1String(Constants::SETTINGS_CATEGORY_CORE_ICON));
 }
 
 QWidget *ShortcutSettings::widget()
@@ -316,15 +355,21 @@ void ShortcutSettings::finish()
     delete m_widget;
 }
 
+ShortcutItem *shortcutItem(QTreeWidgetItem *treeItem)
+{
+    if (!treeItem)
+        return nullptr;
+    return treeItem->data(0, Qt::UserRole).value<ShortcutItem *>();
+}
+
 void ShortcutSettingsWidget::handleCurrentCommandChanged(QTreeWidgetItem *current)
 {
-    if (!current || !current->data(0, Qt::UserRole).isValid()) {
+    ShortcutItem *scitem = shortcutItem(current);
+    if (!scitem) {
         m_shortcutEdit->clear();
         m_warningLabel->clear();
         m_shortcutBox->setEnabled(false);
-        return;
     } else {
-        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
         setKeySequence(scitem->m_key);
         markCollisions(scitem);
         m_shortcutBox->setEnabled(true);
@@ -335,12 +380,9 @@ bool ShortcutSettingsWidget::validateShortcutEdit() const
 {
     m_warningLabel->clear();
     QTreeWidgetItem *current = commandList()->currentItem();
-    if (!current || !current->data(0, Qt::UserRole).isValid())
+    ShortcutItem *item = shortcutItem(current);
+    if (!item)
         return true;
-
-    ShortcutItem *item = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
-    QTC_ASSERT(item, return true);
-
     bool valid = false;
 
     const QString text = m_shortcutEdit->text().trimmed();
@@ -357,10 +399,12 @@ bool ShortcutSettingsWidget::validateShortcutEdit() const
         valid = !that->markCollisions(item);
         if (!valid) {
             m_warningLabel->setText(
-                        tr("Key sequence has potential conflicts. <a href=\"#conflicts\">Show.</a>"));
+                tr("Key sequence has potential conflicts. <a href=\"#conflicts\">Show.</a>"));
+        } else if (isTextKeySequence(currentKey)) {
+            m_warningLabel->setText(tr("Key sequence will not work in editor."));
         }
     } else {
-        m_warningLabel->setText(tr("Invalid key sequence."));
+        m_warningLabel->setText(m_warningLabel->text() + tr("Invalid key sequence."));
     }
     return valid;
 }
@@ -369,14 +413,13 @@ bool ShortcutSettingsWidget::filterColumn(const QString &filterString, QTreeWidg
                                           int column) const
 {
     QString text;
+    ShortcutItem *scitem = shortcutItem(item);
     if (column == item->columnCount() - 1) { // shortcut
         // filter on the shortcut edit text
-        if (!item->data(0, Qt::UserRole).isValid())
+        if (!scitem)
             return true;
-        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(item->data(0, Qt::UserRole));
         text = keySequenceToEditString(scitem->m_key);
-    } else if (column == 0 && item->data(0, Qt::UserRole).isValid()) { // command id
-        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(item->data(0, Qt::UserRole));
+    } else if (column == 0 && scitem) { // command id
         text = scitem->m_cmd->id().toString();
     } else {
         text = item->text(column);
@@ -392,17 +435,16 @@ void ShortcutSettingsWidget::setKeySequence(const QKeySequence &key)
 void ShortcutSettingsWidget::showConflicts()
 {
     QTreeWidgetItem *current = commandList()->currentItem();
-    if (current && current->data(0, Qt::UserRole).isValid()) {
-        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
+    ShortcutItem *scitem = shortcutItem(current);
+    if (scitem)
         setFilterText(keySequenceToEditString(scitem->m_key));
-    }
 }
 
 void ShortcutSettingsWidget::resetToDefault()
 {
     QTreeWidgetItem *current = commandList()->currentItem();
-    if (current && current->data(0, Qt::UserRole).isValid()) {
-        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
+    ShortcutItem *scitem = shortcutItem(current);
+    if (scitem) {
         setKeySequence(scitem->m_cmd->defaultKeySequence());
         foreach (ShortcutItem *item, m_scitems)
             markCollisions(item);
@@ -425,7 +467,7 @@ void ShortcutSettingsWidget::importAction()
                 item->m_key = mapping.value(sid);
                 item->m_item->setText(2, item->m_key.toString(QKeySequence::NativeText));
                 if (item->m_item == commandList()->currentItem())
-                    currentCommandChanged(item->m_item);
+                    emit currentCommandChanged(item->m_item);
 
                 if (item->m_cmd->defaultKeySequence() != item->m_key)
                     setModified(item->m_item, true);
@@ -446,7 +488,7 @@ void ShortcutSettingsWidget::defaultAction()
         item->m_item->setText(2, item->m_key.toString(QKeySequence::NativeText));
         setModified(item->m_item, false);
         if (item->m_item == commandList()->currentItem())
-            currentCommandChanged(item->m_item);
+            emit currentCommandChanged(item->m_item);
     }
 
     foreach (ShortcutItem *item, m_scitems)
@@ -486,8 +528,8 @@ void ShortcutSettingsWidget::initialize()
         if (c->action() && c->action()->isSeparator())
             continue;
 
-        QTreeWidgetItem *item = 0;
-        ShortcutItem *s = new ShortcutItem;
+        QTreeWidgetItem *item = nullptr;
+        auto s = new ShortcutItem;
         m_scitems << s;
         item = new QTreeWidgetItem;
         s->m_cmd = c;
@@ -498,7 +540,7 @@ void ShortcutSettingsWidget::initialize()
         const QString section = identifier.left(pos);
         const QString subId = identifier.mid(pos + 1);
         if (!sections.contains(section)) {
-            QTreeWidgetItem *categoryItem = new QTreeWidgetItem(commandList(), QStringList() << section);
+            QTreeWidgetItem *categoryItem = new QTreeWidgetItem(commandList(), QStringList(section));
             QFont f = categoryItem->font(0);
             f.setBold(true);
             categoryItem->setFont(0, f);
@@ -514,7 +556,7 @@ void ShortcutSettingsWidget::initialize()
         if (s->m_cmd->defaultKeySequence() != s->m_key)
             setModified(item, true);
 
-        item->setData(0, Qt::UserRole, qVariantFromValue(s));
+        item->setData(0, Qt::UserRole, QVariant::fromValue(s));
 
         markCollisions(s);
     }
@@ -552,7 +594,7 @@ bool ShortcutSettingsWidget::markCollisions(ShortcutItem *item)
     }
     item->m_item->setForeground(2, hasCollision
                                 ? Utils::creatorTheme()->color(Utils::Theme::TextColorError)
-                                : commandList()->palette().foreground());
+                                : commandList()->palette().windowText());
     return hasCollision;
 }
 
