@@ -26,6 +26,7 @@
 #include "findtoolbar.h"
 #include "ifindfilter.h"
 #include "findplugin.h"
+#include "optionspopup.h"
 
 #include <coreplugin/coreicons.h>
 #include <coreplugin/coreplugin.h>
@@ -85,8 +86,9 @@ FindToolBar::FindToolBar(CurrentDocumentFind *currentDocumentFind)
 
     connect(m_ui.findEdit, &Utils::FancyLineEdit::editingFinished,
             this, &FindToolBar::invokeResetIncrementalSearch);
+    connect(m_ui.findEdit, &Utils::FancyLineEdit::textChanged,
+            this, &FindToolBar::updateFindReplaceEnabled);
 
-    setLightColoredIcon(false);
     connect(m_ui.close, &QToolButton::clicked,
             this, &FindToolBar::hideAndResetFocus);
 
@@ -98,8 +100,6 @@ FindToolBar::FindToolBar(CurrentDocumentFind *currentDocumentFind)
     m_ui.findEdit->setButtonVisible(Utils::FancyLineEdit::Left, true);
     m_ui.findEdit->setFiltering(true);
     m_ui.findEdit->setPlaceholderText(QString());
-    m_ui.findEdit->setOkColor(Utils::creatorTheme()->color(Utils::Theme::TextColorNormal));
-    m_ui.findEdit->setErrorColor(Utils::creatorTheme()->color(Utils::Theme::TextColorError));
     m_ui.findEdit->button(Utils::FancyLineEdit::Left)->setFocusPolicy(Qt::TabFocus);
     m_ui.findEdit->setValidationFunction([this](Utils::FancyLineEdit *, QString *) {
                                              return m_lastResult != IFindSupport::NotFound;
@@ -117,8 +117,7 @@ FindToolBar::FindToolBar(CurrentDocumentFind *currentDocumentFind)
             this, &FindToolBar::invokeFindEnter, Qt::QueuedConnection);
     connect(m_ui.replaceEdit, &Utils::FancyLineEdit::returnPressed,
             this, &FindToolBar::invokeReplaceEnter, Qt::QueuedConnection);
-    connect(m_findCompleter,
-            static_cast<void (QCompleter::*)(const QModelIndex &)>(&QCompleter::activated),
+    connect(m_findCompleter, QOverload<const QModelIndex &>::of(&QCompleter::activated),
             this, &FindToolBar::findCompleterActivated);
 
     auto shiftEnterAction = new QAction(m_ui.findEdit);
@@ -302,15 +301,17 @@ FindToolBar::FindToolBar(CurrentDocumentFind *currentDocumentFind)
     connect(m_currentDocumentFind, &CurrentDocumentFind::candidateChanged,
             this, &FindToolBar::adaptToCandidate);
     connect(m_currentDocumentFind, &CurrentDocumentFind::changed,
-            this, &FindToolBar::updateGlobalActions);
+            this, &FindToolBar::updateActions);
     connect(m_currentDocumentFind, &CurrentDocumentFind::changed, this, &FindToolBar::updateToolBar);
-    updateGlobalActions();
+    updateActions();
     updateToolBar();
 
     m_findIncrementalTimer.setSingleShot(true);
     m_findStepTimer.setSingleShot(true);
     connect(&m_findIncrementalTimer, &QTimer::timeout, this, &FindToolBar::invokeFindIncremental);
     connect(&m_findStepTimer, &QTimer::timeout, this, &FindToolBar::invokeFindStep);
+
+    setLightColoredIcon(isLightColored());
 }
 
 FindToolBar::~FindToolBar() = default;
@@ -381,7 +382,7 @@ bool FindToolBar::eventFilter(QObject *obj, QEvent *event)
 
 void FindToolBar::adaptToCandidate()
 {
-    updateGlobalActions();
+    updateActions();
     if (findToolBarPlaceHolder() == FindToolBarPlaceHolder::getCurrent()) {
         m_currentDocumentFind->acceptCandidate();
         if (isVisible() && m_currentDocumentFind->isEnabled())
@@ -389,37 +390,26 @@ void FindToolBar::adaptToCandidate()
     }
 }
 
-void FindToolBar::updateGlobalActions()
+void FindToolBar::updateActions()
 {
     IFindSupport *candidate = m_currentDocumentFind->candidate();
     bool enabled = (candidate != nullptr);
-    bool replaceEnabled = enabled && candidate->supportsReplace();
     m_findInDocumentAction->setEnabled(enabled || (toolBarHasFocus() && isEnabled()));
     m_findNextSelectedAction->setEnabled(enabled);
     m_findPreviousSelectedAction->setEnabled(enabled);
     if (m_enterFindStringAction)
         m_enterFindStringAction->setEnabled(enabled);
-    m_findNextAction->setEnabled(enabled);
-    m_findPreviousAction->setEnabled(enabled);
-    m_replaceAction->setEnabled(replaceEnabled);
-    m_replaceNextAction->setEnabled(replaceEnabled);
-    m_replacePreviousAction->setEnabled(replaceEnabled);
-    m_replaceAllAction->setEnabled(replaceEnabled);
+    updateFindReplaceEnabled();
 }
 
 void FindToolBar::updateToolBar()
 {
     bool enabled = m_currentDocumentFind->isEnabled();
     bool replaceEnabled = enabled && m_currentDocumentFind->supportsReplace();
-    bool showAllControls = canShowAllControls(replaceEnabled);
-
-    m_localFindNextAction->setEnabled(enabled);
-    m_localFindPreviousAction->setEnabled(enabled);
-
-    m_localReplaceAction->setEnabled(replaceEnabled);
-    m_localReplacePreviousAction->setEnabled(replaceEnabled);
-    m_localReplaceNextAction->setEnabled(replaceEnabled);
-    m_localReplaceAllAction->setEnabled(replaceEnabled);
+    const ControlStyle style = controlStyle(replaceEnabled);
+    const bool showAllControls = style != ControlStyle::Hidden;
+    setFindButtonStyle(style == ControlStyle::Text ? Qt::ToolButtonTextOnly
+                                                   : Qt::ToolButtonIconOnly);
 
     m_caseSensitiveAction->setEnabled(enabled);
     m_wholeWordAction->setEnabled(enabled);
@@ -431,7 +421,9 @@ void FindToolBar::updateToolBar()
     m_ui.findLabel->setVisible(showAllControls);
     m_ui.findEdit->setEnabled(enabled);
     m_ui.findEdit->setPlaceholderText(showAllControls ? QString() : tr("Search for..."));
+    m_ui.findPreviousButton->setEnabled(enabled);
     m_ui.findPreviousButton->setVisible(showAllControls);
+    m_ui.findNextButton->setEnabled(enabled);
     m_ui.findNextButton->setVisible(showAllControls);
     m_ui.horizontalSpacer->changeSize((showAllControls ? FINDBUTTON_SPACER_WIDTH : 0), 0,
                                       QSizePolicy::Expanding, QSizePolicy::Ignored);
@@ -668,7 +660,8 @@ void FindToolBar::findFlagsChanged()
 
 void FindToolBar::findEditButtonClicked()
 {
-    auto popup = new OptionsPopup(m_ui.findEdit);
+    auto popup = new OptionsPopup(m_ui.findEdit, {Constants::CASE_SENSITIVE, Constants::WHOLE_WORDS,
+                                                  Constants::REGULAR_EXPRESSIONS, Constants::PRESERVE_CASE});
     popup->show();
 }
 
@@ -759,24 +752,39 @@ bool FindToolBar::toolBarHasFocus() const
     return QApplication::focusWidget() == focusWidget();
 }
 
-bool FindToolBar::canShowAllControls(bool replaceIsVisible) const
+FindToolBar::ControlStyle FindToolBar::controlStyle(bool replaceIsVisible)
 {
-    int fullWidth = width();
-    int findFixedWidth = m_ui.findLabel->sizeHint().width()
-            + m_ui.findNextButton->sizeHint().width()
-            + m_ui.findPreviousButton->sizeHint().width()
-            + FINDBUTTON_SPACER_WIDTH
-            + m_ui.close->sizeHint().width();
-    if (fullWidth - findFixedWidth < MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT)
-        return false;
+    const Qt::ToolButtonStyle currentFindButtonStyle = m_ui.findNextButton->toolButtonStyle();
+    const int fullWidth = width();
+    const auto findWidth = [this] {
+        return m_ui.findLabel->sizeHint().width() + m_ui.findNextButton->sizeHint().width()
+               + m_ui.findPreviousButton->sizeHint().width() + FINDBUTTON_SPACER_WIDTH
+               + m_ui.close->sizeHint().width();
+    };
+    setFindButtonStyle(Qt::ToolButtonTextOnly);
+    const int findWithTextWidth = findWidth();
+    setFindButtonStyle(Qt::ToolButtonIconOnly);
+    const int findWithIconsWidth = findWidth();
+    setFindButtonStyle(currentFindButtonStyle);
+    if (fullWidth - findWithIconsWidth < MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT)
+        return ControlStyle::Hidden;
+    if (fullWidth - findWithTextWidth < MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT)
+        return ControlStyle::Icon;
     if (!replaceIsVisible)
-        return true;
-    int replaceFixedWidth = m_ui.replaceLabel->sizeHint().width()
+        return ControlStyle::Text;
+    const int replaceFixedWidth = m_ui.replaceLabel->sizeHint().width()
             + m_ui.replaceButton->sizeHint().width()
             + m_ui.replaceNextButton->sizeHint().width()
             + m_ui.replaceAllButton->sizeHint().width()
             + m_ui.advancedButton->sizeHint().width();
-    return fullWidth - replaceFixedWidth >= MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT;
+    return fullWidth - replaceFixedWidth >= MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT ? ControlStyle::Text
+                                                                             : ControlStyle::Hidden;
+}
+
+void FindToolBar::setFindButtonStyle(Qt::ToolButtonStyle style)
+{
+    m_ui.findPreviousButton->setToolButtonStyle(style);
+    m_ui.findNextButton->setToolButtonStyle(style);
 }
 
 /*!
@@ -970,81 +978,42 @@ void FindToolBar::setBackward(bool backward)
 
 void FindToolBar::setLightColoredIcon(bool lightColored)
 {
-    if (lightColored) {
-        m_ui.findNextButton->setIcon(Utils::Icons::NEXT.icon());
-        m_ui.findPreviousButton->setIcon(Utils::Icons::PREV.icon());
-        m_ui.close->setIcon(Utils::Icons::CLOSE_FOREGROUND.icon());
-    } else {
-        m_ui.findNextButton->setIcon(Utils::Icons::NEXT_TOOLBAR.icon());
-        m_ui.findPreviousButton->setIcon(Utils::Icons::PREV_TOOLBAR.icon());
-        m_ui.close->setIcon(Utils::Icons::CLOSE_TOOLBAR.icon());
+    m_localFindNextAction->setIcon(lightColored ? Utils::Icons::NEXT.icon()
+                                                : Utils::Icons::NEXT_TOOLBAR.icon());
+    m_localFindPreviousAction->setIcon(lightColored ? Utils::Icons::PREV.icon()
+                                                    : Utils::Icons::PREV_TOOLBAR.icon());
+    m_ui.close->setIcon(lightColored ? Utils::Icons::CLOSE_FOREGROUND.icon()
+                                     : Utils::Icons::CLOSE_TOOLBAR.icon());
+}
+
+void FindToolBar::updateFindReplaceEnabled()
+{
+    bool enabled = !getFindText().isEmpty();
+    if (enabled != m_findEnabled) {
+        m_localFindNextAction->setEnabled(enabled);
+        m_localFindPreviousAction->setEnabled(enabled);
+        m_findEnabled = enabled;
     }
+    m_findNextAction->setEnabled(enabled && m_findInDocumentAction->isEnabled());
+    m_findPreviousAction->setEnabled(enabled && m_findInDocumentAction->isEnabled());
+
+    updateReplaceEnabled();
 }
 
-OptionsPopup::OptionsPopup(QWidget *parent)
-    : QWidget(parent, Qt::Popup)
+void FindToolBar::updateReplaceEnabled()
 {
-    setAttribute(Qt::WA_DeleteOnClose);
-    auto layout = new QVBoxLayout(this);
-    layout->setContentsMargins(2, 2, 2, 2);
-    layout->setSpacing(2);
-    setLayout(layout);
-    QCheckBox *firstCheckBox = createCheckboxForCommand(Constants::CASE_SENSITIVE);
-    layout->addWidget(firstCheckBox);
-    layout->addWidget(createCheckboxForCommand(Constants::WHOLE_WORDS));
-    layout->addWidget(createCheckboxForCommand(Constants::REGULAR_EXPRESSIONS));
-    layout->addWidget(createCheckboxForCommand(Constants::PRESERVE_CASE));
-    firstCheckBox->setFocus();
-    move(parent->mapToGlobal(QPoint(0, -sizeHint().height())));
-}
+    const bool enabled = m_findEnabled && m_currentDocumentFind->supportsReplace();
+    m_localReplaceAction->setEnabled(enabled);
+    m_localReplaceAllAction->setEnabled(enabled);
+    m_localReplaceNextAction->setEnabled(enabled);
+    m_localReplacePreviousAction->setEnabled(enabled);
 
-bool OptionsPopup::event(QEvent *ev)
-{
-    if (ev->type() == QEvent::ShortcutOverride) {
-        auto ke = static_cast<QKeyEvent *>(ev);
-        if (ke->key() == Qt::Key_Escape && !ke->modifiers()) {
-            ev->accept();
-            return true;
-        }
-    }
-    return QWidget::event(ev);
-}
-
-bool OptionsPopup::eventFilter(QObject *obj, QEvent *ev)
-{
-    auto checkbox = qobject_cast<QCheckBox *>(obj);
-    if (ev->type() == QEvent::KeyPress && checkbox) {
-        auto ke = static_cast<QKeyEvent *>(ev);
-        if (!ke->modifiers() && (ke->key() == Qt::Key_Enter || ke->key() == Qt::Key_Return)) {
-            checkbox->click();
-            ev->accept();
-            return true;
-        }
-    }
-    return QWidget::eventFilter(obj, ev);
-}
-
-void OptionsPopup::actionChanged()
-{
-    auto action = qobject_cast<QAction *>(sender());
-    QTC_ASSERT(action, return);
-    QCheckBox *checkbox = m_checkboxMap.value(action);
-    QTC_ASSERT(checkbox, return);
-    checkbox->setEnabled(action->isEnabled());
-}
-
-QCheckBox *OptionsPopup::createCheckboxForCommand(Id id)
-{
-    QAction *action = ActionManager::command(id)->action();
-    QCheckBox *checkbox = new QCheckBox(action->text());
-    checkbox->setToolTip(action->toolTip());
-    checkbox->setChecked(action->isChecked());
-    checkbox->setEnabled(action->isEnabled());
-    checkbox->installEventFilter(this); // enter key handling
-    QObject::connect(checkbox, &QCheckBox::clicked, action, &QAction::setChecked);
-    QObject::connect(action, &QAction::changed, this, &OptionsPopup::actionChanged);
-    m_checkboxMap.insert(action, checkbox);
-    return checkbox;
+    IFindSupport *currentCandidate = m_currentDocumentFind->candidate();
+    bool globalsEnabled = currentCandidate ? currentCandidate->supportsReplace() : false;
+    m_replaceAction->setEnabled(globalsEnabled);
+    m_replaceAllAction->setEnabled(globalsEnabled);
+    m_replaceNextAction->setEnabled(globalsEnabled);
+    m_replacePreviousAction->setEnabled(globalsEnabled);
 }
 
 } // namespace Internal
