@@ -69,7 +69,7 @@ enum RegisterDataRole
 class RegisterDelegate : public QItemDelegate
 {
 public:
-    RegisterDelegate() {}
+    RegisterDelegate() = default;
 
     QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &,
         const QModelIndex &index) const override
@@ -80,7 +80,7 @@ public:
             lineEdit->setFrame(false);
             return lineEdit;
         }
-        return 0;
+        return nullptr;
     }
 
     void setEditorData(QWidget *editor, const QModelIndex &index) const override
@@ -119,7 +119,7 @@ public:
                 painter->setPen(lightColor);
             // FIXME: performance? this changes only on real font changes.
             QFontMetrics fm(option.font);
-            int charWidth = qMax(fm.width(QLatin1Char('x')), fm.width(QLatin1Char('0')));
+            int charWidth = qMax(fm.horizontalAdvance('x'), fm.horizontalAdvance('0'));
             QString str = index.data(Qt::DisplayRole).toString();
             int x = option.rect.x();
             bool light = !paintRed;
@@ -174,26 +174,26 @@ void Register::guessMissingData()
 
 static QString subTypeName(RegisterKind kind, int size, RegisterFormat format)
 {
-    QString name(QLatin1Char('['));
+    QString name('[');
 
     switch (kind) {
-        case IntegerRegister: name += QLatin1Char('i'); break;
-        case FloatRegister: name += QLatin1Char('f'); break;
+        case IntegerRegister: name += 'i'; break;
+        case FloatRegister: name += 'f'; break;
         default: break;
     }
 
     name += QString::number(size);
 
     switch (format) {
-        case BinaryFormat: name += QLatin1Char('b'); break;
-        case OctalFormat: name += QLatin1Char('o'); break;
-        case DecimalFormat: name += QLatin1Char('u'); break;
-        case SignedDecimalFormat: name += QLatin1Char('s'); break;
-        case HexadecimalFormat: name += QLatin1Char('x'); break;
-        case CharacterFormat: name += QLatin1Char('c'); break;
+        case BinaryFormat: name += 'b'; break;
+        case OctalFormat: name += 'o'; break;
+        case DecimalFormat: name += 'u'; break;
+        case SignedDecimalFormat: name += 's'; break;
+        case HexadecimalFormat: name += 'x'; break;
+        case CharacterFormat: name += 'c'; break;
     }
 
-    name += QLatin1Char(']');
+    name += ']';
 
     return name;
 }
@@ -428,13 +428,13 @@ public:
             appendChild(new RegisterEditItem(i, subKind, subSize, format));
     }
 
-    QVariant data(int column, int role) const;
+    QVariant data(int column, int role) const override;
 
-    Qt::ItemFlags flags(int column) const
+    Qt::ItemFlags flags(int column) const override
     {
         //return column == 1 ? Qt::ItemIsSelectable|Qt::ItemIsEnabled|Qt::ItemIsEditable
         //                   : Qt::ItemIsSelectable|Qt::ItemIsEnabled;
-        Q_UNUSED(column);
+        Q_UNUSED(column)
         return Qt::ItemIsSelectable|Qt::ItemIsEnabled;
     }
 
@@ -464,7 +464,7 @@ public:
 };
 
 RegisterItem::RegisterItem(DebuggerEngine *engine, const Register &reg)
-    : m_engine(engine), m_reg(reg), m_changed(true)
+    : m_engine(engine), m_reg(reg)
 {
     if (m_reg.kind == UnknownRegister)
         m_reg.guessMissingData();
@@ -492,7 +492,7 @@ Qt::ItemFlags RegisterItem::flags(int column) const
 {
     const Qt::ItemFlags notEditable = Qt::ItemIsSelectable|Qt::ItemIsEnabled;
     // Can edit registers if they are hex numbers and not arrays.
-    if (column == 1) //  && IntegerWatchLineEdit::isUnsignedHexNumber(QLatin1String(m_reg.display)))
+    if (column == 1) //  && IntegerWatchLineEdit::isUnsignedHexNumber(m_reg.display))
         return notEditable | Qt::ItemIsEditable;
     return notEditable;
 }
@@ -676,11 +676,11 @@ bool RegisterHandler::setData(const QModelIndex &idx, const QVariant &data, int
 
 bool RegisterHandler::contextMenuEvent(const ItemViewEvent &ev)
 {
-    const bool actionsEnabled = m_engine->debuggerActionsEnabled();
     const DebuggerState state = m_engine->state();
+    const bool actionsEnabled = m_engine->debuggerActionsEnabled();
 
-    RegisterItem *registerItem = itemForIndexAtLevel<1>(ev.index());
-    RegisterSubItem *registerSubItem = itemForIndexAtLevel<2>(ev.index());
+    RegisterItem *registerItem = itemForIndexAtLevel<1>(ev.sourceModelIndex());
+    RegisterSubItem *registerSubItem = itemForIndexAtLevel<2>(ev.sourceModelIndex());
 
     const quint64 address = registerItem ? registerItem->addressValue() : 0;
     const QString registerName = registerItem ? registerItem->m_reg.name : QString();
@@ -742,8 +742,10 @@ bool RegisterHandler::contextMenuEvent(const ItemViewEvent &ev)
               ? registerSubItem->parent()->m_format
               : HexadecimalFormat;
 
-    auto addFormatAction = [this, menu, currentFormat, registerItem](const QString &display, RegisterFormat format) {
-        addCheckableAction(menu, display, registerItem, currentFormat == format, [this, registerItem, format] {
+    auto addFormatAction =
+            [menu, currentFormat, registerItem](const QString &display, RegisterFormat format) {
+        addCheckableAction(menu, display, registerItem, currentFormat == format,
+                           [registerItem, format] {
             registerItem->m_format = format;
             registerItem->update();
         });
@@ -754,7 +756,7 @@ bool RegisterHandler::contextMenuEvent(const ItemViewEvent &ev)
     addFormatAction(tr("Octal"), OctalFormat);
     addFormatAction(tr("Binary"), BinaryFormat);
 
-    menu->addSeparator();
+    Internal::addHideColumnActions(menu, ev.view());
     menu->addAction(action(SettingsDialog));
     menu->popup(ev.globalPos());
     return true;
