@@ -24,18 +24,25 @@
 ****************************************************************************/
 
 #include "diffutils.h"
-#include "differ.h"
 
-#include "texteditor/fontsettings.h"
-#include "utils/asconst.h"
+#include <texteditor/fontsettings.h>
+#include <utils/algorithm.h>
+#include <utils/differ.h>
 
 #include <QFutureInterfaceBase>
 #include <QRegularExpression>
 #include <QStringList>
 #include <QTextStream>
 
+using namespace Utils;
+
 namespace DiffEditor {
 
+int ChunkSelection::selectedRowsCount() const
+{
+    return Utils::toSet(leftSelection).unite(Utils::toSet(rightSelection)).count();
+}
+
 static QList<TextLineData> assemblyRows(const QList<TextLineData> &lines,
                                         const QMap<int, int> &lineSpans)
 {
@@ -54,10 +61,10 @@ static QList<TextLineData> assemblyRows(const QList<TextLineData> &lines,
 static bool lastLinesEqual(const QList<TextLineData> &leftLines,
                            const QList<TextLineData> &rightLines)
 {
-    const bool leftLineEqual = leftLines.count()
+    const bool leftLineEqual = !leftLines.isEmpty()
             ? leftLines.last().text.isEmpty()
             : true;
-    const bool rightLineEqual = rightLines.count()
+    const bool rightLineEqual = !rightLines.isEmpty()
             ? rightLines.last().text.isEmpty()
             : true;
     return leftLineEqual && rightLineEqual;
@@ -84,7 +91,7 @@ static void handleDifference(const QString &text,
                              QList<TextLineData> *lines,
                              int *lineNumber)
 {
-    const QStringList newLines = text.split(QLatin1Char('\n'));
+    const QStringList newLines = text.split('\n');
     for (int line = 0; line < newLines.count(); ++line) {
         const int startPos = line > 0
                 ? -1
@@ -104,7 +111,7 @@ static void handleDifference(const QString &text,
  * The number of equalities on both lists must be the same.
 */
 ChunkData DiffUtils::calculateOriginalData(const QList<Diff> &leftDiffList,
-                                const QList<Diff> &rightDiffList)
+                                           const QList<Diff> &rightDiffList)
 {
     int i = 0;
     int j = 0;
@@ -133,7 +140,7 @@ ChunkData DiffUtils::calculateOriginalData(const QList<Diff> &leftDiffList,
                 : Diff(Diff::Equal);
 
         if (leftDiff.command == Diff::Delete) {
-            if (j == rightDiffList.count() && lastLineEqual && leftDiff.text.startsWith(QLatin1Char('\n')))
+            if (j == rightDiffList.count() && lastLineEqual && leftDiff.text.startsWith('\n'))
                 equalLines.insert(leftLineNumber, rightLineNumber);
             // process delete
             handleDifference(leftDiff.text, &leftLines, &leftLineNumber);
@@ -143,7 +150,7 @@ ChunkData DiffUtils::calculateOriginalData(const QList<Diff> &leftDiffList,
             i++;
         }
         if (rightDiff.command == Diff::Insert) {
-            if (i == leftDiffList.count() && lastLineEqual && rightDiff.text.startsWith(QLatin1Char('\n')))
+            if (i == leftDiffList.count() && lastLineEqual && rightDiff.text.startsWith('\n'))
                 equalLines.insert(leftLineNumber, rightLineNumber);
             // process insert
             handleDifference(rightDiff.text, &rightLines, &rightLineNumber);
@@ -154,12 +161,12 @@ ChunkData DiffUtils::calculateOriginalData(const QList<Diff> &leftDiffList,
         }
         if (leftDiff.command == Diff::Equal && rightDiff.command == Diff::Equal) {
             // process equal
-            const QStringList newLeftLines = leftDiff.text.split(QLatin1Char('\n'));
-            const QStringList newRightLines = rightDiff.text.split(QLatin1Char('\n'));
+            const QStringList newLeftLines = leftDiff.text.split('\n');
+            const QStringList newRightLines = rightDiff.text.split('\n');
 
             int line = 0;
 
-            if (i < leftDiffList.count() || j < rightDiffList.count() || (leftLines.count() && rightLines.count())) {
+            if (i < leftDiffList.count() || j < rightDiffList.count() || (!leftLines.isEmpty() && !rightLines.isEmpty())) {
                 while (line < qMax(newLeftLines.count(), newRightLines.count())) {
                     handleLine(newLeftLines, line, &leftLines, &leftLineNumber);
                     handleLine(newRightLines, line, &rightLines, &rightLineNumber);
@@ -347,9 +354,9 @@ QString DiffUtils::makePatchLine(const QChar &startLineCharacter,
             || addNoNewline; // no addNoNewline case
 
     if (addLine) {
-        line = startLineCharacter + textLine + QLatin1Char('\n');
+        line = startLineCharacter + textLine + '\n';
         if (addNoNewline)
-            line += QLatin1String("\\ No newline at end of file\n");
+            line += "\\ No newline at end of file\n";
     }
 
     return line;
@@ -402,9 +409,9 @@ QString DiffUtils::makePatch(const ChunkData &chunkData,
                                         // ensure we process buffers to the end.
                                         // rowData will be equal
         if (rowData.equal && i != rowToBeSplit) {
-            if (leftBuffer.count()) {
+            if (!leftBuffer.isEmpty()) {
                 for (int j = 0; j < leftBuffer.count(); j++) {
-                    const QString line = makePatchLine(QLatin1Char('-'),
+                    const QString line = makePatchLine('-',
                                               leftBuffer.at(j).text,
                                               lastChunk,
                                               i == chunkData.rows.count()
@@ -417,9 +424,9 @@ QString DiffUtils::makePatch(const ChunkData &chunkData,
                 }
                 leftBuffer.clear();
             }
-            if (rightBuffer.count()) {
+            if (!rightBuffer.isEmpty()) {
                 for (int j = 0; j < rightBuffer.count(); j++) {
-                    const QString line = makePatchLine(QLatin1Char('+'),
+                    const QString line = makePatchLine('+',
                                               rightBuffer.at(j).text,
                                               lastChunk,
                                               i == chunkData.rows.count()
@@ -433,7 +440,7 @@ QString DiffUtils::makePatch(const ChunkData &chunkData,
                 rightBuffer.clear();
             }
             if (i < chunkData.rows.count()) {
-                const QString line = makePatchLine(QLatin1Char(' '),
+                const QString line = makePatchLine(' ',
                                           rowData.rightLine.text,
                                           lastChunk,
                                           i == chunkData.rows.count() - 1);
@@ -453,17 +460,17 @@ QString DiffUtils::makePatch(const ChunkData &chunkData,
         }
     }
 
-    const QString chunkLine = QLatin1String("@@ -")
+    const QString chunkLine = "@@ -"
             + QString::number(chunkData.leftStartingLineNumber + 1)
-            + QLatin1Char(',')
+            + ','
             + QString::number(leftLineCount)
-            + QLatin1String(" +")
+            + " +"
             + QString::number(chunkData.rightStartingLineNumber + 1)
-            + QLatin1Char(',')
+            + ','
             + QString::number(rightLineCount)
-            + QLatin1String(" @@")
+            + " @@"
             + chunkData.contextInfo
-            + QLatin1Char('\n');
+            + '\n';
 
     diffText.prepend(chunkLine);
 
@@ -477,8 +484,8 @@ QString DiffUtils::makePatch(const ChunkData &chunkData,
 {
     QString diffText = makePatch(chunkData, lastChunk);
 
-    const QString rightFileInfo = QLatin1String("+++ ") + rightFileName + QLatin1Char('\n');
-    const QString leftFileInfo = QLatin1String("--- ") + leftFileName + QLatin1Char('\n');
+    const QString rightFileInfo = "+++ " + rightFileName + '\n';
+    const QString leftFileInfo = "--- " + leftFileName + '\n';
 
     diffText.prepend(rightFileInfo);
     diffText.prepend(leftFileInfo);
@@ -568,7 +575,7 @@ static QList<RowData> readLines(QStringRef patch,
 {
     QList<Diff> diffList;
 
-    const QChar newLine = QLatin1Char('\n');
+    const QChar newLine = '\n';
 
     int lastEqual = -1;
     int lastDelete = -1;
@@ -588,7 +595,7 @@ static QList<RowData> readLines(QStringRef patch,
             break;
         }
         const QChar firstCharacter = line.at(0);
-        if (firstCharacter == QLatin1Char('\\')) { // no new line marker
+        if (firstCharacter == '\\') { // no new line marker
             if (!lastChunk) // can only appear in last chunk of the file
                 break;
             if (!diffList.isEmpty()) {
@@ -612,11 +619,11 @@ static QList<RowData> readLines(QStringRef patch,
             }
         } else {
             Diff::Command command = Diff::Equal;
-            if (firstCharacter == QLatin1Char(' ')) { // common line
+            if (firstCharacter == ' ') { // common line
                 command = Diff::Equal;
-            } else if (firstCharacter == QLatin1Char('-')) { // deleted line
+            } else if (firstCharacter == '-') { // deleted line
                 command = Diff::Delete;
-            } else if (firstCharacter == QLatin1Char('+')) { // inserted line
+            } else if (firstCharacter == '+') { // inserted line
                 command = Diff::Insert;
             } else { // no other character may exist as the first character
                 if (lastChunk)
@@ -1025,10 +1032,16 @@ static bool detectIndexAndBinary(QStringRef patch,
     bool hasNewLine;
     *remainingPatch = patch;
 
-    if (remainingPatch->isEmpty() && (fileData->fileOperation == FileData::CopyFile
-                            || fileData->fileOperation == FileData::RenameFile)) {
-        // in case of 100% similarity we don't have more lines in the patch
-        return true;
+    if (remainingPatch->isEmpty()) {
+        switch (fileData->fileOperation) {
+        case FileData::CopyFile:
+        case FileData::RenameFile:
+        case FileData::ChangeMode:
+            // in case of 100% similarity we don't have more lines in the patch
+            return true;
+        default:
+            break;
+        }
     }
 
     QStringRef afterNextLine;
@@ -1068,9 +1081,9 @@ static bool detectIndexAndBinary(QStringRef patch,
     const QString rightFileName = fileData->fileOperation == FileData::DeleteFile
             ? devNull : QLatin1String("b/") + fileData->rightFileInfo.fileName;
 
-    const QString binaryLine = QLatin1String("Binary files ")
-            + leftFileName + QLatin1String(" and ")
-            + rightFileName + QLatin1String(" differ");
+    const QString binaryLine = "Binary files "
+            + leftFileName + " and "
+            + rightFileName + " differ";
 
     if (*remainingPatch == binaryLine) {
         fileData->binaryFiles = true;
@@ -1078,7 +1091,7 @@ static bool detectIndexAndBinary(QStringRef patch,
         return true;
     }
 
-    const QString leftStart = QLatin1String("--- ") + leftFileName;
+    const QString leftStart = "--- " + leftFileName;
     QStringRef afterMinuses;
     // --- leftFileName
     const QStringRef minuses = readLine(*remainingPatch, &afterMinuses, &hasNewLine);
@@ -1088,7 +1101,7 @@ static bool detectIndexAndBinary(QStringRef patch,
     if (!minuses.startsWith(leftStart))
         return false;
 
-    const QString rightStart = QLatin1String("+++ ") + rightFileName;
+    const QString rightStart = "+++ " + rightFileName;
     QStringRef afterPluses;
     // +++ rightFileName
     const QStringRef pluses = readLine(afterMinuses, &afterPluses, &hasNewLine);
@@ -1151,8 +1164,6 @@ static bool detectFileData(QStringRef patch,
 
         QStringRef afterSecondLine;
         const QStringRef secondLine = readLine(afterDiffGit, &afterSecondLine, &hasNewLine);
-        if (!hasNewLine)
-            return false; // we need to have at least one more line
 
         if (secondLine.startsWith(QStringLiteral("new file mode "))) {
             fileData->fileOperation = FileData::NewFile;
@@ -1165,7 +1176,7 @@ static bool detectFileData(QStringRef patch,
             // new mode
             readLine(afterSecondLine, &afterThirdLine, &hasNewLine);
             if (!hasNewLine)
-                return false; // we need to have at least one more line
+                fileData->fileOperation = FileData::ChangeMode;
 
             // TODO: validate new mode line
             *remainingPatch = afterThirdLine;
@@ -1282,7 +1293,7 @@ static QList<FileData> readGitPatch(QStringRef patch, bool *ok,
     QList<FileData> fileDataList;
     readOk = false;
     int i = 0;
-    for (const auto &patchInfo : Utils::asConst(patches)) {
+    for (const auto &patchInfo : qAsConst(patches)) {
         if (jobController) {
             if (jobController->isCanceled())
                 return QList<FileData>();
