@@ -31,12 +31,14 @@
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/documentmanager.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/iwizardfactory.h>
 
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
+#include <utils/icon.h>
 #include <utils/stringutils.h>
 #include <utils/theme/theme.h>
 
@@ -48,6 +50,7 @@
 #include <QHeaderView>
 #include <QHelpEvent>
 #include <QLabel>
+#include <QMenu>
 #include <QPainter>
 #include <QToolTip>
 #include <QTreeView>
@@ -81,7 +84,6 @@ QVariant ProjectModel::data(const QModelIndex &index, int role) const
     case Qt::DisplayRole:
         return data.second;
     case Qt::ToolTipRole:
-        return data.first;
     case FilePathRole:
         return data.first;
     case PrettyFilePathRole:
@@ -127,12 +129,18 @@ ProjectWelcomePage::ProjectWelcomePage()
         auto act = new QAction(tr("Open Session #%1").arg(i), this);
         Command *cmd = ActionManager::registerAction(act, sessionBase.withSuffix(i), welcomeContext);
         cmd->setDefaultKeySequence(QKeySequence((useMacShortcuts ? tr("Ctrl+Meta+%1") : tr("Ctrl+Alt+%1")).arg(i)));
-        connect(act, &QAction::triggered, this, [this, i] { openSessionAt(i - 1); });
+        connect(act, &QAction::triggered, this, [this, i] {
+            if (i <= m_sessionModel->rowCount())
+                openSessionAt(i - 1);
+        });
 
         act = new QAction(tr("Open Recent Project #%1").arg(i), this);
         cmd = ActionManager::registerAction(act, projectBase.withSuffix(i), welcomeContext);
         cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+Shift+%1").arg(i)));
-        connect(act, &QAction::triggered, this, [this, i] { openProjectAt(i - 1); });
+        connect(act, &QAction::triggered, this, [this, i] {
+            if (i <= m_projectModel->rowCount(QModelIndex()))
+                openProjectAt(i - 1);
+        });
     }
 }
 
@@ -334,7 +342,7 @@ public:
             };
             for (int i = 0; i < 3; ++i) {
                 const QString &action = actions.at(i);
-                const int ww = fm.width(action);
+                const int ww = fm.horizontalAdvance(action);
                 const QRect actionRect(xx, yy - 10, ww, 15);
                 const bool isForcedDisabled = (i != 0 && sessionName == "default");
                 const bool isActive = actionRect.contains(mousePos) && !isForcedDisabled;
@@ -379,7 +387,7 @@ public:
                     m_expandedSessions.removeOne(sessionName);
                 else
                     m_expandedSessions.append(sessionName);
-                model->layoutChanged({QPersistentModelIndex(idx)});
+                emit model->layoutChanged({QPersistentModelIndex(idx)});
                 return true;
             }
             if (button == Qt::LeftButton) {
@@ -393,12 +401,12 @@ public:
                 else if (m_activeActionRects[1].contains(pos))
                     sessionModel->renameSession(ICore::mainWindow(), sessionName);
                 else if (m_activeActionRects[2].contains(pos))
-                    sessionModel->deleteSession(sessionName);
+                    sessionModel->deleteSessions(QStringList(sessionName));
                 return true;
             }
         }
         if (ev->type() == QEvent::MouseMove) {
-            model->layoutChanged({QPersistentModelIndex(idx)}); // Somewhat brutish.
+            emit model->layoutChanged({QPersistentModelIndex(idx)}); // Somewhat brutish.
             //update(option.rect);
             return true;
         }
@@ -469,11 +477,12 @@ public:
         QString projectName = idx.data(Qt::DisplayRole).toString();
         QString projectPath = idx.data(ProjectModel::FilePathRole).toString();
         QFontMetrics fm(sizedFont(13, option.widget));
-        int width = std::max(fm.width(projectName), fm.width(projectPath)) + 36;
+        int width = std::max(fm.horizontalAdvance(projectName),
+                             fm.horizontalAdvance(projectPath)) + 36;
         return QSize(width, 48);
     }
 
-    bool editorEvent(QEvent *ev, QAbstractItemModel *,
+    bool editorEvent(QEvent *ev, QAbstractItemModel *model,
         const QStyleOptionViewItem &, const QModelIndex &idx) final
     {
         if (ev->type() == QEvent::MouseButtonRelease) {
@@ -484,6 +493,28 @@ public:
                 ProjectExplorerPlugin::openProjectWelcomePage(projectFile);
                 return true;
             }
+            if (button == Qt::RightButton) {
+                QMenu contextMenu;
+                QAction *action = new QAction(
+                    ProjectWelcomePage::tr("Remove Project from Recent Projects"));
+                const auto projectModel = qobject_cast<ProjectModel *>(model);
+                contextMenu.addAction(action);
+                connect(action, &QAction::triggered, [idx, projectModel](){
+                    const QString projectFile = idx.data(ProjectModel::FilePathRole).toString();
+                    const QString displayName = idx.data(Qt::DisplayRole).toString();
+                    ProjectExplorerPlugin::removeFromRecentProjects(projectFile, displayName);
+                    projectModel->resetProjects();
+                });
+                contextMenu.addSeparator();
+                action = new QAction(ProjectWelcomePage::tr("Clear Recent Project List"));
+                connect(action, &QAction::triggered, [projectModel]() {
+                    ProjectExplorerPlugin::clearRecentProjects();
+                    projectModel->resetProjects();
+                });
+                contextMenu.addAction(action);
+                contextMenu.exec(mouseEvent->globalPos());
+                return true;
+            }
         }
         return false;
     }
@@ -492,9 +523,10 @@ public:
 class TreeView : public QTreeView
 {
 public:
-    TreeView(QWidget *parent)
+    TreeView(QWidget *parent, const QString &name)
         : QTreeView(parent)
     {
+        setObjectName(name);
         header()->hide();
         setMouseTracking(true); // To enable hover.
         setIndentation(0);
@@ -528,13 +560,19 @@ public:
         if (!projectWelcomePage->m_projectModel)
             projectWelcomePage->m_projectModel = new ProjectModel(this);
 
+        auto manageSessionsButton = new WelcomePageButton(this);
+        manageSessionsButton->setText(ProjectWelcomePage::tr("Manage"));
+        const Icon icon({{":/utils/images/settings.png", Theme::Welcome_ForegroundSecondaryColor}}, Icon::Tint);
+        manageSessionsButton->setIcon(icon.pixmap());
+        manageSessionsButton->setOnClicked([] { ProjectExplorerPlugin::showSessionManager(); });
+
         auto newButton = new WelcomePageButton(this);
-        newButton->setText(ProjectWelcomePage::tr("New Project"));
+        newButton->setText(ProjectWelcomePage::tr("New"));
         newButton->setIcon(pixmap("new", Theme::Welcome_ForegroundSecondaryColor));
         newButton->setOnClicked([] { ProjectExplorerPlugin::openNewProjectDialog(); });
 
         auto openButton = new WelcomePageButton(this);
-        openButton->setText(ProjectWelcomePage::tr("Open Project"));
+        openButton->setText(ProjectWelcomePage::tr("Open"));
         openButton->setIcon(pixmap("open", Theme::Welcome_ForegroundSecondaryColor));
         openButton->setOnClicked([] { ProjectExplorerPlugin::openOpenProjectDialog(); });
 
@@ -544,15 +582,15 @@ public:
 
         auto recentProjectsLabel = new QLabel(this);
         recentProjectsLabel->setFont(sizedFont(16, this));
-        recentProjectsLabel->setText(ProjectWelcomePage::tr("Recent Projects"));
+        recentProjectsLabel->setText(ProjectWelcomePage::tr("Projects"));
 
-        auto sessionsList = new TreeView(this);
+        auto sessionsList = new TreeView(this, "Sessions");
         sessionsList->setModel(projectWelcomePage->m_sessionModel);
         sessionsList->header()->setSectionHidden(1, true); // The "last modified" column.
         sessionsList->setItemDelegate(&m_sessionDelegate);
         sessionsList->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
 
-        auto projectsList = new TreeView(this);
+        auto projectsList = new TreeView(this, "Recent Projects");
         projectsList->setUniformRowHeights(true);
         projectsList->setModel(projectWelcomePage->m_projectModel);
         projectsList->setItemDelegate(&m_projectDelegate);
@@ -560,11 +598,17 @@ public:
 
         auto hbox11 = new QHBoxLayout;
         hbox11->setContentsMargins(0, 0, 0, 0);
-        hbox11->addWidget(newButton);
+        hbox11->addWidget(sessionsLabel);
+        hbox11->addSpacing(16);
+        hbox11->addWidget(manageSessionsButton);
         hbox11->addStretch(1);
 
         auto hbox21 = new QHBoxLayout;
         hbox21->setContentsMargins(0, 0, 0, 0);
+        hbox21->addWidget(recentProjectsLabel);
+        hbox21->addSpacing(16);
+        hbox21->addWidget(newButton);
+        hbox21->addSpacing(16);
         hbox21->addWidget(openButton);
         hbox21->addStretch(1);
 
@@ -573,16 +617,13 @@ public:
         vbox1->addStrut(200);
         vbox1->addItem(hbox11);
         vbox1->addSpacing(16);
-        vbox1->addWidget(sessionsLabel);
-        vbox1->addSpacing(21);
         vbox1->addWidget(sessionsList);
 
         auto vbox2 = new QVBoxLayout;
         vbox2->setContentsMargins(0, 0, 0, 0);
+        vbox1->addStrut(200);
         vbox2->addItem(hbox21);
         vbox2->addSpacing(16);
-        vbox2->addWidget(recentProjectsLabel);
-        vbox2->addSpacing(21);
         vbox2->addWidget(projectsList);
 
         auto hbox = new QHBoxLayout(this);
