@@ -63,12 +63,12 @@ TerminalAspect::TerminalAspect()
             this, &TerminalAspect::calculateUseTerminal);
 }
 
-void TerminalAspect::addToConfigurationLayout(QFormLayout *layout)
+void TerminalAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!m_checkBox);
-    m_checkBox = new QCheckBox(tr("Run in terminal"), layout->parentWidget());
+    m_checkBox = new QCheckBox(tr("Run in terminal"));
     m_checkBox->setChecked(m_useTerminal);
-    layout->addRow(QString(), m_checkBox);
+    builder.addItems(QString(), m_checkBox.data());
     connect(m_checkBox.data(), &QAbstractButton::clicked, this, [this] {
         m_userSet = true;
         m_useTerminal = m_checkBox->isChecked();
@@ -103,7 +103,7 @@ void TerminalAspect::calculateUseTerminal()
     switch (ProjectExplorerPlugin::projectExplorerSettings().terminalMode) {
     case Internal::TerminalMode::On: useTerminal = true; break;
     case Internal::TerminalMode::Off: useTerminal = false; break;
-    case Internal::TerminalMode::Smart: useTerminal = m_useTerminalHint;
+    default: useTerminal = m_useTerminalHint;
     }
     if (m_useTerminal != useTerminal) {
         m_useTerminal = useTerminal;
@@ -140,14 +140,14 @@ WorkingDirectoryAspect::WorkingDirectoryAspect()
     setSettingsKey("RunConfiguration.WorkingDirectory");
 }
 
-void WorkingDirectoryAspect::addToConfigurationLayout(QFormLayout *layout)
+void WorkingDirectoryAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!m_chooser);
-    m_chooser = new PathChooser(layout->parentWidget());
+    m_chooser = new PathChooser;
     m_chooser->setHistoryCompleter(settingsKey());
     m_chooser->setExpectedKind(Utils::PathChooser::Directory);
     m_chooser->setPromptDialogTitle(tr("Select Working Directory"));
-    m_chooser->setBaseFileName(m_defaultWorkingDirectory);
+    m_chooser->setBaseDirectory(m_defaultWorkingDirectory);
     m_chooser->setFileName(m_workingDirectory.isEmpty() ? m_defaultWorkingDirectory : m_workingDirectory);
     connect(m_chooser.data(), &PathChooser::pathChanged, this,
             [this]() {
@@ -155,7 +155,7 @@ void WorkingDirectoryAspect::addToConfigurationLayout(QFormLayout *layout)
                 m_resetButton->setEnabled(m_workingDirectory != m_defaultWorkingDirectory);
             });
 
-    m_resetButton = new QToolButton(layout->parentWidget());
+    m_resetButton = new QToolButton;
     m_resetButton->setToolTip(tr("Reset to Default"));
     m_resetButton->setIcon(Utils::Icons::RESET.icon());
     connect(m_resetButton.data(), &QAbstractButton::clicked, this, &WorkingDirectoryAspect::resetPath);
@@ -168,10 +168,7 @@ void WorkingDirectoryAspect::addToConfigurationLayout(QFormLayout *layout)
         m_chooser->setEnvironment(m_envAspect->environment());
     }
 
-    auto hbox = new QHBoxLayout;
-    hbox->addWidget(m_chooser);
-    hbox->addWidget(m_resetButton);
-    layout->addRow(tr("Working directory:"), hbox);
+    builder.addItems(tr("Working directory:"), m_chooser.data(), m_resetButton.data());
 }
 
 void WorkingDirectoryAspect::acquaintSiblings(const ProjectConfigurationAspects &siblings)
@@ -237,7 +234,7 @@ void WorkingDirectoryAspect::setDefaultWorkingDirectory(const FilePath &defaultW
     Utils::FilePath oldDefaultDir = m_defaultWorkingDirectory;
     m_defaultWorkingDirectory = defaultWorkingDir;
     if (m_chooser)
-        m_chooser->setBaseFileName(m_defaultWorkingDirectory);
+        m_chooser->setBaseDirectory(m_defaultWorkingDirectory);
 
     if (m_workingDirectory.isEmpty() || m_workingDirectory == oldDefaultDir) {
         if (m_chooser)
@@ -337,16 +334,17 @@ QWidget *ArgumentsAspect::setupChooser()
     return m_chooser.data();
 }
 
-void ArgumentsAspect::addToConfigurationLayout(QFormLayout *layout)
+void ArgumentsAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!m_chooser && !m_multiLineChooser && !m_multiLineButton);
+    builder.addItem(tr("Command line arguments:"));
 
     const auto container = new QWidget;
     const auto containerLayout = new QHBoxLayout(container);
     containerLayout->setContentsMargins(0, 0, 0, 0);
     containerLayout->addWidget(setupChooser());
     m_multiLineButton = new ExpandButton;
-    m_multiLineButton->setToolTip(tr("Toggle multi-line mode"));
+    m_multiLineButton->setToolTip(tr("Toggle multi-line mode."));
     m_multiLineButton->setChecked(m_multiLine);
     connect(m_multiLineButton, &QCheckBox::clicked, this, [this](bool checked) {
         if (m_multiLine == checked)
@@ -371,7 +369,8 @@ void ArgumentsAspect::addToConfigurationLayout(QFormLayout *layout)
     });
     containerLayout->addWidget(m_multiLineButton);
     containerLayout->setAlignment(m_multiLineButton, Qt::AlignTop);
-    layout->addRow(tr("Command line arguments:"), container);
+
+    builder.addItem(container);
 }
 
 /*!
@@ -437,7 +436,8 @@ void ExecutableAspect::makeOverridable(const QString &overridingKey, const QStri
     m_alternativeExecutable->setDisplayStyle(BaseStringAspect::LineEditDisplay);
     m_alternativeExecutable->setLabelText(tr("Alternate executable on device:"));
     m_alternativeExecutable->setSettingsKey(overridingKey);
-    m_alternativeExecutable->makeCheckable(tr("Use this command instead"), useOverridableKey);
+    m_alternativeExecutable->makeCheckable(BaseStringAspect::CheckBoxPlacement::Right,
+                                           tr("Use this command instead"), useOverridableKey);
     connect(m_alternativeExecutable, &BaseStringAspect::changed,
             this, &ExecutableAspect::changed);
 }
@@ -450,11 +450,11 @@ FilePath ExecutableAspect::executable() const
     return m_executable.filePath();
 }
 
-void ExecutableAspect::addToConfigurationLayout(QFormLayout *layout)
+void ExecutableAspect::addToLayout(LayoutBuilder &builder)
 {
-    m_executable.addToConfigurationLayout(layout);
+    m_executable.addToLayout(builder);
     if (m_alternativeExecutable)
-        m_alternativeExecutable->addToConfigurationLayout(layout);
+        m_alternativeExecutable->addToLayout(builder.startNewRow());
 }
 
 void ExecutableAspect::setLabelText(const QString &labelText)
@@ -469,7 +469,7 @@ void ExecutableAspect::setPlaceHolderText(const QString &placeHolderText)
 
 void ExecutableAspect::setExecutable(const FilePath &executable)
 {
-   m_executable.setValue(executable.toString());
+   m_executable.setFilePath(executable);
    m_executable.setShowToolTipOnLabel(true);
 }
 
@@ -502,12 +502,15 @@ UseLibraryPathsAspect::UseLibraryPathsAspect()
 {
     setId("UseLibraryPath");
     setSettingsKey("RunConfiguration.UseLibrarySearchPath");
-    if (HostOsInfo::isMacHost())
-        setLabel(tr("Add build library search path to DYLD_LIBRARY_PATH and DYLD_FRAMEWORK_PATH"));
-    else if (HostOsInfo::isWindowsHost())
-        setLabel(tr("Add build library search path to PATH"));
-    else
-        setLabel(tr("Add build library search path to LD_LIBRARY_PATH"));
+    if (HostOsInfo::isMacHost()) {
+        setLabel(tr("Add build library search path to DYLD_LIBRARY_PATH and DYLD_FRAMEWORK_PATH"),
+                 LabelPlacement::AtCheckBox);
+    } else if (HostOsInfo::isWindowsHost()) {
+        setLabel(tr("Add build library search path to PATH"), LabelPlacement::AtCheckBox);
+    } else {
+        setLabel(tr("Add build library search path to LD_LIBRARY_PATH"),
+                 LabelPlacement::AtCheckBox);
+    }
     setValue(ProjectExplorerPlugin::projectExplorerSettings().addLibraryPathsToRunEnv);
 }
 
@@ -519,7 +522,8 @@ UseDyldSuffixAspect::UseDyldSuffixAspect()
 {
     setId("UseDyldSuffix");
     setSettingsKey("RunConfiguration.UseDyldImageSuffix");
-    setLabel(tr("Use debug version of frameworks (DYLD_IMAGE_SUFFIX=_debug)"));
+    setLabel(tr("Use debug version of frameworks (DYLD_IMAGE_SUFFIX=_debug)"),
+             LabelPlacement::AtCheckBox);
 }
 
 } // namespace ProjectExplorer
