@@ -33,9 +33,8 @@
 #include <coreplugin/id.h>
 
 namespace ProjectExplorer {
-class Kit;
-class Project;
-}
+class Target;
+} // namespace ProjectExplorer
 
 namespace QmlDesigner {
 
@@ -50,56 +49,47 @@ public:
         UserSpacePuppet
     };
 
-    PuppetCreator(ProjectExplorer::Kit *kit,
-                  ProjectExplorer::Project *project,
-                  const QString &qtCreatorVersion,
-                  const Model *model);
+    PuppetCreator(ProjectExplorer::Target *target, const Model *model);
 
-    ~PuppetCreator();
-
-    void createPuppetExecutableIfMissing();
+    void createQml2PuppetExecutableIfMissing();
 
     QProcess *createPuppetProcess(const QString &puppetMode,
                                   const QString &socketToken,
                                   QObject *handlerObject,
                                   const char *outputSlot,
-                                  const char *finishSlot) const;
-
-    QString compileLog() const;
+                                  const char *finishSlot,
+                                  const QStringList &customOptions = {}) const;
 
     void setQrcMappingString(const QString qrcMapping);
 
     static QString defaultPuppetToplevelBuildDirectory();
     static QString defaultPuppetFallbackDirectory();
+    static QString qmlPuppetFallbackDirectory(const DesignerSettings &settings);
 protected:
     bool build(const QString &qmlPuppetProjectFilePath) const;
 
-    void createQml2PuppetExecutableIfMissing();
 
     QString qmlPuppetToplevelBuildDirectory() const;
-    QString qmlPuppetFallbackDirectory() const;
     QString qmlPuppetDirectory(PuppetType puppetPathType) const;
     QString qml2PuppetPath(PuppetType puppetType) const;
 
     bool startBuildProcess(const QString &buildDirectoryPath,
                            const QString &command,
                            const QStringList &processArguments = QStringList(),
-                           PuppetBuildProgressDialog *progressDialog = 0) const;
+                           PuppetBuildProgressDialog *progressDialog = nullptr) const;
     static QString puppetSourceDirectoryPath();
     static QString qml2PuppetProjectFile();
-    static QString qmlPuppetProjectFile();
 
     bool checkPuppetIsReady(const QString &puppetPath) const;
-    bool checkQml2PuppetIsReady() const;
     bool qtIsSupported() const;
-    static bool checkPuppetVersion(const QString &qmlPuppetPath);
     QProcess *puppetProcess(const QString &puppetPath,
                             const QString &workingDirectory,
                             const QString &puppetMode,
                             const QString &socketToken,
                             QObject *handlerObject,
                             const char *outputSlot,
-                            const char *finishSlot) const;
+                            const char *finishSlot,
+                            const QStringList &customOptions) const;
 
     QProcessEnvironment processEnvironment() const;
 
@@ -114,17 +104,15 @@ protected:
     QString getStyleConfigFileName() const;
 
 private:
-    QString m_qtCreatorVersion;
     mutable QString m_compileLog;
-    ProjectExplorer::Kit *m_kit;
+    ProjectExplorer::Target *m_target = nullptr;
     PuppetType m_availablePuppetType;
     static QHash<Core::Id, PuppetType> m_qml2PuppetForKitPuppetHash;
-    const Model *m_model;
+    const Model *m_model = nullptr;
 #ifndef QMLDESIGNER_TEST
     const DesignerSettings m_designerSettings;
 #endif
     QString m_qrcMapping;
-    ProjectExplorer::Project *m_currentProject = nullptr;
 };
 
 } // namespace QmlDesigner
