@@ -133,7 +133,7 @@ void TransitionItem::createGrabbers()
     if (m_cornerGrabbers.count() != m_cornerPoints.count()) {
         int selectedGrabberIndex = m_cornerGrabbers.indexOf(m_selectedCornerGrabber);
 
-        if (m_cornerGrabbers.count() > 0) {
+        if (!m_cornerGrabbers.isEmpty()) {
             qDeleteAll(m_cornerGrabbers);
             m_cornerGrabbers.clear();
         }
@@ -158,7 +158,7 @@ void TransitionItem::createGrabbers()
 
 void TransitionItem::removeGrabbers()
 {
-    if (m_cornerGrabbers.count() > 0) {
+    if (!m_cornerGrabbers.isEmpty()) {
         qDeleteAll(m_cornerGrabbers);
         m_cornerGrabbers.clear();
     }
@@ -267,7 +267,7 @@ void TransitionItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
             m_cornerPoints.append(p);
             snapToAnyPoint(m_cornerPoints.count() - 1, p);
 
-            if (m_cornerGrabbers.count() > 0) {
+            if (!m_cornerGrabbers.isEmpty()) {
                 auto corner = new CornerGrabberItem(this);
                 corner->setGrabberType(CornerGrabberItem::Circle);
                 corner->setPos(p);
@@ -306,11 +306,19 @@ void TransitionItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
                     QPointF intersPoint;
                     QLineF line2(p, p + QPointF(SELECTION_DISTANCE, SELECTION_DISTANCE));
                     line2.setAngle(line.angle() + 90);
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
                     if (line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection)
+#else
+                    if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection)
+#endif
                         sel = true;
                     else {
                         line2.setAngle(line.angle() - 90);
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
                         sel = line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection;
+#else
+                        sel = line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection;
+#endif
                     }
 
                     if (sel)
@@ -607,7 +615,7 @@ void TransitionItem::connectToTopItem(const QPointF &pos, TransitionPoint tp, It
 
     // First try to find parentItem
     QList<QGraphicsItem*> items = scene()->items(p);
-    if (items.count() > 0) {
+    if (!items.isEmpty()) {
         for (int i = 0; i < items.count(); ++i) {
             ItemType type = ItemType(items[i]->type());
             if ((targetType == UnknownType && type >= FinalStateType) || type >= StateType) {
@@ -649,7 +657,7 @@ void TransitionItem::connectToTopItem(const QPointF &pos, TransitionPoint tp, It
         removeGrabbers();
         if (m_startItem == m_endItem && cornerPoints == 2) {
             setTagValue("type", "internal");
-            setEndItem(0);
+            setEndItem(nullptr);
             m_targetType = InternalNoTarget;
         }
 
@@ -789,7 +797,7 @@ QPointF TransitionItem::findIntersectionPoint(ConnectableItem *item, const QLine
 
     // Find intersection point between line and target item
     QPolygonF itemPolygon = item->polygonShape();
-    if (itemPolygon.count() > 0) {
+    if (!itemPolygon.isEmpty()) {
         QPointF intersectPoint;
         QPointF p1 = itemPolygon.at(0) + item->scenePos();
         QPointF p2;
@@ -797,7 +805,11 @@ QPointF TransitionItem::findIntersectionPoint(ConnectableItem *item, const QLine
         for (int i = 1; i < itemPolygon.count(); ++i) {
             p2 = itemPolygon.at(i) + item->scenePos();
             checkLine = QLineF(p1, p2);
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
             if (checkLine.intersect(line, &intersectPoint) == QLineF::BoundedIntersection)
+#else
+            if (checkLine.intersects(line, &intersectPoint) == QLineF::BoundedIntersection)
+#endif
                 return intersectPoint;
             p1 = p2;
         }
@@ -1083,11 +1095,19 @@ bool TransitionItem::containsScenePoint(const QPointF &p) const
         QPointF intersPoint;
         QLineF line2(pp, pp + QPointF(SELECTION_DISTANCE, SELECTION_DISTANCE));
         line2.setAngle(line.angle() + 90);
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
         if (line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection) {
+#else
+        if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection) {
+#endif
             return true;
         } else {
             line2.setAngle(line.angle() - 90);
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
             if (line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection)
+#else
+            if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection)
+#endif
                 return true;
         }
     }
@@ -1169,7 +1189,7 @@ void TransitionItem::storeValues(bool block)
 
 void TransitionItem::updateUIProperties()
 {
-    if (tag() != 0 && isActiveScene())
+    if (tag() && isActiveScene())
         storeValues();
 }
 
@@ -1180,9 +1200,9 @@ void TransitionItem::updateTargetType()
 
     TransitionTargetType type = ExternalTarget;
 
-    if (m_startItem != 0 && m_startItem == m_endItem)
+    if (m_startItem && m_startItem == m_endItem)
         type = InternalSameTarget;
-    else if (m_startItem != 0 && !m_endItem) {
+    else if (m_startItem && !m_endItem) {
         if (m_movingLastPoint) {
             type = ExternalNoTarget;
         } else {
