@@ -68,7 +68,7 @@ public:
     Type type() const { return m_type; }
     virtual void apply(ValueHandler *) = 0;
     virtual ValuePtr clone() const = 0;
-    virtual CodeLocation location() const { return CodeLocation(); }
+    virtual CodeLocation location() const { return {}; }
 
     Item *definingItem() const;
     virtual void setDefiningItem(Item *item);
@@ -115,10 +115,10 @@ class JSSourceValue : public Value
 
 public:
     static JSSourceValuePtr QBS_AUTOTEST_EXPORT create(bool createdByPropertiesBlock = false);
-    ~JSSourceValue();
+    ~JSSourceValue() override;
 
-    void apply(ValueHandler *handler) { handler->handle(this); }
-    ValuePtr clone() const;
+    void apply(ValueHandler *handler) override { handler->handle(this); }
+    ValuePtr clone() const override;
 
     void setSourceCode(const QStringRef &sourceCode) { m_sourceCode = sourceCode; }
     const QStringRef &sourceCode() const { return m_sourceCode; }
@@ -127,7 +127,7 @@ public:
     void setLocation(int line, int column);
     int line() const { return m_line; }
     int column() const { return m_column; }
-    CodeLocation location() const;
+    CodeLocation location() const override;
 
     void setFile(const FileContextPtr &file) { m_file = file; }
     const FileContextPtr &file() const { return m_file; }
@@ -151,14 +151,14 @@ public:
         struct PropertyData
         {
             PropertyData() = default;
-            PropertyData(const QString &v, const CodeLocation &l) : value(v), location(l) {}
+            PropertyData(QString v, const CodeLocation &l) : value(std::move(v)), location(l) {}
             QString value;
             CodeLocation location;
         };
 
-        Alternative() { }
-        Alternative(const PropertyData &c, const PropertyData &o, const JSSourceValuePtr &v)
-            : condition(c), overrideListProperties(o), value(v) {}
+        Alternative() = default;
+        Alternative(PropertyData c, PropertyData o, JSSourceValuePtr v)
+            : condition(std::move(c)), overrideListProperties(std::move(o)), value(std::move(v)) {}
         Alternative clone() const
         {
             return Alternative(condition, overrideListProperties,
@@ -175,7 +175,7 @@ public:
     void addAlternative(const Alternative &alternative) { m_alternatives.push_back(alternative); }
     void clearAlternatives();
 
-    void setDefiningItem(Item *item);
+    void setDefiningItem(Item *item) override;
 
 private:
     QStringRef m_sourceCode;
@@ -207,12 +207,12 @@ private:
 
 class VariantValue : public Value
 {
-    VariantValue(const QVariant &v);
+    VariantValue(QVariant v);
 public:
     static VariantValuePtr create(const QVariant &v = QVariant());
 
-    void apply(ValueHandler *handler) { handler->handle(this); }
-    ValuePtr clone() const;
+    void apply(ValueHandler *handler) override { handler->handle(this); }
+    ValuePtr clone() const override;
 
     const QVariant &value() const { return m_value; }
 
