@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -55,6 +50,9 @@
 
 using namespace CPlusPlus;
 
+// to use in tests directly w/o convertion to int
+Q_DECLARE_METATYPE(Function::RefQualifier)
+
 class tst_Semantic: public QObject
 {
     Q_OBJECT
@@ -116,19 +114,19 @@ public:
 
         virtual void report(int /*level*/,
                             const StringLiteral *fileName,
-                            unsigned line, unsigned column,
+                            int line, int column,
                             const char *format, va_list ap)
         {
             ++errorCount;
 
 #ifndef NO_PARSER_OR_SEMANTIC_ERROR_MESSAGES
-            qDebug() << fileName->chars()<<':'<<line<<':'<<column<<' '<<QString().vsprintf(format, ap);
+            qDebug() << fileName->chars()<<':'<<line<<':'<<column<<' ' << QString::vasprintf(format, ap);
 #else
-            Q_UNUSED(fileName);
-            Q_UNUSED(line);
-            Q_UNUSED(column);
-            Q_UNUSED(format);
-            Q_UNUSED(ap);
+            Q_UNUSED(fileName)
+            Q_UNUSED(line)
+            Q_UNUSED(column)
+            Q_UNUSED(format)
+            Q_UNUSED(ap)
 #endif
         }
     };
@@ -144,6 +142,7 @@ public:
         features.qtMocRunEnabled = qtMocRun;
         features.qtKeywordsEnabled = qtMocRun;
         features.cxx11Enabled = enableCxx11;
+        features.cxxEnabled = true;
         diag.errorCount = 0; // reset the error count.
         TranslationUnit *unit = parse(source, TranslationUnit::ParseTranlationUnit, features);
         QSharedPointer<Document> doc(new Document(unit));
@@ -155,6 +154,8 @@ public:
 private slots:
     void function_declaration_1();
     void function_declaration_2();
+    void function_declaration_ref_qualifier_data();
+    void function_declaration_ref_qualifier();
     void function_definition_1();
     void nested_class_1();
     void alias_declaration_1();
@@ -184,13 +185,20 @@ private slots:
     void lambda_2();
 
     void diagnostic_error();
+
+    void enum_constantValue1();
+    void enum_constantValue2();
+    void enum_constantValue3();
+    void enum_constantValue4();
+    void enum_constantValue5();
+    void enum_constantValueNegative();
 };
 
 void tst_Semantic::function_declaration_1()
 {
     QSharedPointer<Document> doc = document("void foo();");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -199,7 +207,8 @@ void tst_Semantic::function_declaration_1()
     Function *funTy = declTy->asFunctionType();
     QVERIFY(funTy);
     QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 0U);
+    QCOMPARE(funTy->argumentCount(), 0);
+    QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
     QVERIFY(decl->name()->isNameId());
     const Identifier *funId = decl->name()->asNameId()->identifier();
@@ -213,7 +222,7 @@ void tst_Semantic::function_declaration_2()
 {
     QSharedPointer<Document> doc = document("void foo(const QString &s);");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -222,7 +231,8 @@ void tst_Semantic::function_declaration_2()
     Function *funTy = declTy->asFunctionType();
     QVERIFY(funTy);
     QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 1U);
+    QCOMPARE(funTy->argumentCount(), 1);
+    QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
     // check the formal argument.
     Argument *arg = funTy->argumentAt(0)->asArgument();
@@ -259,16 +269,114 @@ void tst_Semantic::function_declaration_2()
     QCOMPARE(foo, QByteArray("foo"));
 }
 
+void tst_Semantic::function_declaration_ref_qualifier_data()
+{
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<bool>("success");
+    QTest::addColumn<Function::RefQualifier>("refQualifier");
+
+    QTest::newRow("no")
+            << "void f();" << true << Function::NoRefQualifier;
+
+    QTest::newRow("no_const")
+            << "void f() const;" << true << Function::NoRefQualifier;
+
+    QTest::newRow("no_const_noexcept")
+            << "void f() const noexcept;" << true << Function::NoRefQualifier;
+
+    QTest::newRow("lvalue")
+            << "void f() &;" << true << Function::LvalueRefQualifier;
+
+    QTest::newRow("lvalue_const")
+            << "void f() const &;" << true << Function::LvalueRefQualifier;
+
+    QTest::newRow("lvalue_const_noexcept")
+            << "void f() const & noexcept;" << true << Function::LvalueRefQualifier;
+
+    QTest::newRow("rvalue")
+            << "void f() &&;" << true << Function::RvalueRefQualifier;
+
+    QTest::newRow("rvalue_const")
+            << "void f() const &&;" << true << Function::RvalueRefQualifier;
+
+    QTest::newRow("rvalue_const_noexcept")
+            << "void f() const && noexcept;" << true << Function::RvalueRefQualifier;
+
+    QTest::newRow("lvalue_more_spaces")
+            << "void f()  const    &;" << true << Function::LvalueRefQualifier;
+
+    QTest::newRow("rvalue_more_spaces")
+            << "void f()       &&    noexcept;" << true << Function::RvalueRefQualifier;
+
+    QTest::newRow("lvalue_more_newline")
+            << "void f() const\n&;" << true << Function::LvalueRefQualifier;
+
+    QTest::newRow("rvalue_more_newline")
+            << "void f() const\n&&;" << true << Function::RvalueRefQualifier;
+
+    QTest::newRow("lvalue_no_space")
+            << "void f() const& noexcept;" << true << Function::LvalueRefQualifier;
+
+    QTest::newRow("rvalue_no_space")
+            << "void f() const&& noexcept;" << true << Function::RvalueRefQualifier;
+
+    QTest::newRow("lvalue_before_const")
+            << "void f() & const;" << false << Function::NoRefQualifier;
+
+    QTest::newRow("rvalue_before_const")
+            << "void f() && const;" << false << Function::NoRefQualifier;
+
+    QTest::newRow("lvalue_after_noexcept")
+            << "void f() const noexcept &;" << false << Function::NoRefQualifier;
+
+    QTest::newRow("rvalue_after_noexcept")
+            << "void f() const noexcept &&;" << false << Function::NoRefQualifier;
+
+    QTest::newRow("lvalue_double")
+            << "void f() const & & noexcept;" << false << Function::NoRefQualifier;
+
+    QTest::newRow("rvalue_double")
+            << "void f() const && && noexcept;" << false << Function::NoRefQualifier;
+}
+
+void tst_Semantic::function_declaration_ref_qualifier()
+{
+    QFETCH(QString, code);
+    QFETCH(bool, success);
+    QFETCH(Function::RefQualifier, refQualifier);
+
+    QSharedPointer<Document> doc = document(code.toUtf8(), false, false, true);
+    if (!success) {
+        QVERIFY(doc->errorCount > 0);
+        return;
+    }
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+
+    Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
+    QVERIFY(decl);
+
+    FullySpecifiedType declTy = decl->type();
+    Function *funTy = declTy->asFunctionType();
+    QVERIFY(funTy);
+    QVERIFY(funTy->returnType()->isVoidType());
+    QCOMPARE(funTy->argumentCount(), 0);
+
+    // check the ref-qualifier
+    QCOMPARE(funTy->refQualifier(), refQualifier);
+}
+
 void tst_Semantic::function_definition_1()
 {
     QSharedPointer<Document> doc = document("void foo() {}");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Function *funTy = doc->globals->memberAt(0)->asFunction();
     QVERIFY(funTy);
     QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 0U);
+    QCOMPARE(funTy->argumentCount(), 0);
+    QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
     QVERIFY(funTy->name()->isNameId());
     const Identifier *funId = funTy->name()->asNameId()->identifier();
@@ -290,7 +398,7 @@ void tst_Semantic::nested_class_1()
 "};\n"
     );
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 2U);
+    QCOMPARE(doc->globals->memberCount(), 2);
 
     Class *classObject = doc->globals->memberAt(0)->asClass();
     QVERIFY(classObject);
@@ -299,8 +407,8 @@ void tst_Semantic::nested_class_1()
     QVERIFY(classObjectNameId);
     const Identifier *objectId = classObjectNameId->identifier();
     QCOMPARE(QByteArray(objectId->chars(), objectId->size()), QByteArray("Object"));
-    QCOMPARE(classObject->baseClassCount(), 0U);
-    QCOMPARE(classObject->memberCount(), 2U);
+    QCOMPARE(classObject->baseClassCount(), 0);
+    QCOMPARE(classObject->memberCount(), 2);
 
     Class *classObjectData = doc->globals->memberAt(1)->asClass();
     QVERIFY(classObjectData);
@@ -331,7 +439,7 @@ void tst_Semantic::alias_declaration_1()
                 , false, false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl->name());
@@ -356,11 +464,11 @@ void tst_Semantic::typedef_1()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 3U);
+    QCOMPARE(doc->globals->memberCount(), 3);
 
     Class *anonStruct = doc->globals->memberAt(0)->asClass();
     QVERIFY(anonStruct);
-    QCOMPARE(anonStruct->memberCount(), 2U);
+    QCOMPARE(anonStruct->memberCount(), 2);
 
     Declaration *typedefPointDecl = doc->globals->memberAt(1)->asDeclaration();
     QVERIFY(typedefPointDecl);
@@ -385,11 +493,11 @@ void tst_Semantic::typedef_2()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 3U);
+    QCOMPARE(doc->globals->memberCount(), 3);
 
     Class *_pointStruct= doc->globals->memberAt(0)->asClass();
     QVERIFY(_pointStruct);
-    QCOMPARE(_pointStruct->memberCount(), 2U);
+    QCOMPARE(_pointStruct->memberCount(), 2);
 
     Declaration *typedefPointDecl = doc->globals->memberAt(1)->asDeclaration();
     QVERIFY(typedefPointDecl);
@@ -410,11 +518,11 @@ void tst_Semantic::typedef_3()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 2U);
+    QCOMPARE(doc->globals->memberCount(), 2);
 
     Class *_pointStruct= doc->globals->memberAt(0)->asClass();
     QVERIFY(_pointStruct);
-    QCOMPARE(_pointStruct->memberCount(), 2U);
+    QCOMPARE(_pointStruct->memberCount(), 2);
 
     Declaration *typedefPointDecl = doc->globals->memberAt(1)->asDeclaration();
     QVERIFY(typedefPointDecl);
@@ -431,14 +539,14 @@ void tst_Semantic::const_1()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
     QVERIFY(decl->type()->isFunctionType());
     Function *funTy = decl->type()->asFunctionType();
     QVERIFY(funTy->returnType()->isIntegerType());
-    QCOMPARE(funTy->argumentCount(), 1U);
+    QCOMPARE(funTy->argumentCount(), 1);
     Argument *arg = funTy->argumentAt(0)->asArgument();
     QVERIFY(arg);
     QVERIFY(! arg->type().isConst());
@@ -454,14 +562,14 @@ void tst_Semantic::const_2()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
     QVERIFY(decl->type()->isFunctionType());
     Function *funTy = decl->type()->asFunctionType();
     QVERIFY(funTy->returnType()->isIntegerType());
-    QCOMPARE(funTy->argumentCount(), 1U);
+    QCOMPARE(funTy->argumentCount(), 1);
     Argument *arg = funTy->argumentAt(0)->asArgument();
     QVERIFY(arg);
     QVERIFY(arg->type().isConst());
@@ -474,7 +582,7 @@ void tst_Semantic::pointer_to_function_1()
 {
     QSharedPointer<Document> doc = document("void (*QtSomething)();");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -496,7 +604,7 @@ void tst_Semantic::template_instance_1()
 {
     QSharedPointer<Document> doc = document("template <typename _Tp> class QList { void append(const _Tp &value); };");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Template *templ = doc->globals->memberAt(0)->asTemplate();
     QVERIFY(templ);
@@ -504,7 +612,7 @@ void tst_Semantic::template_instance_1()
     Declaration *decl = templ->memberAt(1)->asClass()->memberAt(0)->asDeclaration();
     QVERIFY(decl);
 
-    FullySpecifiedType templArgs[] = { control->integerType(IntegerType::Int) };
+    FullySpecifiedType templArgs[] = {control->integerType(IntegerType::Int)};
     const Name *templId = control->templateNameId(control->identifier("QList"), false, templArgs, 1);
 
     FullySpecifiedType genTy = DeprecatedGenTemplateInstance::instantiate(templId, decl, control);
@@ -526,7 +634,7 @@ void tst_Semantic::expression_under_cursor_1()
     QTextCursor tc(&textDocument);
     tc.movePosition(QTextCursor::End);
 
-    ExpressionUnderCursor expressionUnderCursor;
+    ExpressionUnderCursor expressionUnderCursor(LanguageFeatures::defaultFeatures());
     const QString expression = expressionUnderCursor(tc);
 
     QCOMPARE(expression, QString("bar"));
@@ -542,7 +650,7 @@ void tst_Semantic::bracketed_expression_under_cursor_1()
     QTextCursor tc(&textDocument);
     tc.movePosition(QTextCursor::End);
 
-    ExpressionUnderCursor expressionUnderCursor;
+    ExpressionUnderCursor expressionUnderCursor(LanguageFeatures::defaultFeatures());
     const QString expression = expressionUnderCursor(tc);
 
     QCOMPARE(expression, QString("i"));
@@ -558,7 +666,7 @@ void tst_Semantic::bracketed_expression_under_cursor_2()
     QTextCursor tc(&textDocument);
     tc.movePosition(QTextCursor::End);
 
-    ExpressionUnderCursor expressionUnderCursor;
+    ExpressionUnderCursor expressionUnderCursor(LanguageFeatures::defaultFeatures());
     const QString expression = expressionUnderCursor(tc);
 
     QCOMPARE(expression, plainText);
@@ -574,7 +682,7 @@ void tst_Semantic::bracketed_expression_under_cursor_3()
     QTextCursor tc(&textDocument);
     tc.movePosition(QTextCursor::End);
 
-    ExpressionUnderCursor expressionUnderCursor;
+    ExpressionUnderCursor expressionUnderCursor(LanguageFeatures::defaultFeatures());
     const QString expression = expressionUnderCursor(tc);
 
     QCOMPARE(expression, QString("[receiver message"));
@@ -590,7 +698,7 @@ void tst_Semantic::bracketed_expression_under_cursor_4()
     QTextCursor tc(&textDocument);
     tc.movePosition(QTextCursor::End);
 
-    ExpressionUnderCursor expressionUnderCursor;
+    ExpressionUnderCursor expressionUnderCursor(LanguageFeatures::defaultFeatures());
     const QString expression = expressionUnderCursor(tc);
 
     QCOMPARE(expression, QString("i"));
@@ -608,17 +716,17 @@ void tst_Semantic::objcClass_1()
                                             true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 2U);
+    QCOMPARE(doc->globals->memberCount(), 2);
 
     ObjCClass *iface = doc->globals->memberAt(0)->asObjCClass();
     QVERIFY(iface);
     QVERIFY(iface->isInterface());
-    QCOMPARE(iface->memberCount(), 2U);
+    QCOMPARE(iface->memberCount(), 2);
 
     ObjCClass *impl = doc->globals->memberAt(1)->asObjCClass();
     QVERIFY(impl);
     QVERIFY(!impl->isInterface());
-    QCOMPARE(impl->memberCount(), 3U);
+    QCOMPARE(impl->memberCount(), 3);
 
     ObjCMethod *allocMethod = impl->memberAt(0)->asObjCMethod();
     QVERIFY(allocMethod);
@@ -642,19 +750,19 @@ void tst_Semantic::objcSelector_1()
                                             true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     ObjCClass *iface = doc->globals->memberAt(0)->asObjCClass();
     QVERIFY(iface);
     QVERIFY(iface->isInterface());
-    QCOMPARE(iface->memberCount(), 1U);
+    QCOMPARE(iface->memberCount(), 1);
 
     Declaration *decl = iface->memberAt(0)->asDeclaration();
     QVERIFY(decl);
     QVERIFY(decl->name());
     const SelectorNameId *selId = decl->name()->asSelectorNameId();
     QVERIFY(selId);
-    QCOMPARE(selId->nameCount(), 3U);
+    QCOMPARE(selId->nameCount(), 3);
     QCOMPARE(selId->nameAt(0)->identifier()->chars(), "a");
     QCOMPARE(selId->nameAt(1)->identifier()->chars(), "b");
     QCOMPARE(selId->nameAt(2)->identifier()->chars(), "c");
@@ -688,11 +796,11 @@ void tst_Semantic::objcSelector_2()
                                             true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     ObjCClass *iface = doc->globals->memberAt(0)->asObjCClass();
     QVERIFY(iface);
-    QCOMPARE(iface->memberCount(), 1U);
+    QCOMPARE(iface->memberCount(), 1);
 
     QList<ObjCSelectorAST*>selectors = CollectSelectors(doc->unit)();
     QCOMPARE(selectors.size(), 2);
@@ -702,7 +810,7 @@ void tst_Semantic::objcSelector_2()
 
     const SelectorNameId *selId = sel->name->asSelectorNameId();
     QVERIFY(selId);
-    QCOMPARE(selId->nameCount(), 3U);
+    QCOMPARE(selId->nameCount(), 3);
     QCOMPARE(selId->nameAt(0)->identifier()->chars(), "a");
     QCOMPARE(selId->nameAt(1)->identifier()->chars(), "b");
     QCOMPARE(selId->nameAt(2)->identifier()->chars(), "c");
@@ -719,7 +827,7 @@ void tst_Semantic::q_enum_1()
                                             false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     QVERIFY(doc->unit);
     TranslationUnitAST *xUnit = doc->unit->ast()->asTranslationUnit();
     QVERIFY(xUnit);
@@ -743,7 +851,7 @@ void tst_Semantic::lambda_1()
                                             "}\n", false, false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 }
 
 void tst_Semantic::lambda_2()
@@ -756,15 +864,15 @@ void tst_Semantic::lambda_2()
                 , false, false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Class *A = doc->globals->memberAt(0)->asClass();
     QVERIFY(A);
-    QCOMPARE(A->memberCount(), 1U);
+    QCOMPARE(A->memberCount(), 1);
     Declaration *d = A->memberAt(0)->asDeclaration();
     QCOMPARE(d->name()->identifier()->chars(), "f");
     Function *ty = d->type()->asFunctionType();
     QVERIFY(ty);
-    QCOMPARE(ty->argumentCount(), 1U);
+    QCOMPARE(ty->argumentCount(), 1);
     Argument *arg = ty->argumentAt(0)->asArgument();
     QVERIFY(arg);
     const StringLiteral *init = arg->initializer();
@@ -780,7 +888,157 @@ void tst_Semantic::diagnostic_error()
                                             false, false);
 
     QCOMPARE(doc->errorCount, 1U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+}
+
+namespace {
+void testEnumaratorDeclarator(Enum *e, int enumDeclIndex, const char *expectedConstantValue)
+{
+    Declaration *enumMemberDeclaration = e->memberAt(enumDeclIndex)->asDeclaration();
+    QVERIFY(enumMemberDeclaration);
+    EnumeratorDeclaration *enumeratorDeclaration = enumMemberDeclaration->asEnumeratorDeclarator();
+    QVERIFY(enumeratorDeclaration);
+    if (const StringLiteral *constantValue = enumeratorDeclaration->constantValue())
+        QCOMPARE(constantValue->chars(), expectedConstantValue);
+    else
+        QVERIFY(!expectedConstantValue);
+}
+} // anonymous
+
+void tst_Semantic::enum_constantValue1()
+{
+    QSharedPointer<Document> doc = document("\n"
+                                            "enum {\n"
+                                            "E1,\n"
+                                            "E2,\n"
+                                            "E3\n"
+                                            "};\n"
+                                            );
+
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+    Enum *e = doc->globals->memberAt(0)->asEnum();
+    QVERIFY(e);
+    QCOMPARE(e->memberCount(), 3);
+
+    testEnumaratorDeclarator(e, 0, "0");
+    testEnumaratorDeclarator(e, 1, "1");
+    testEnumaratorDeclarator(e, 2, "2");
+}
+
+void tst_Semantic::enum_constantValue2()
+{
+    QSharedPointer<Document> doc = document("\n"
+                                            "enum {\n"
+                                            "E1=10,\n"
+                                            "E2,\n"
+                                            "E3\n"
+                                            "};\n"
+                                            );
+
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+    Enum *e = doc->globals->memberAt(0)->asEnum();
+    QVERIFY(e);
+    QCOMPARE(e->memberCount(), 3);
+
+    testEnumaratorDeclarator(e, 0, "10");
+    testEnumaratorDeclarator(e, 1, "11");
+    testEnumaratorDeclarator(e, 2, "12");
+}
+
+void tst_Semantic::enum_constantValue3()
+{
+    QSharedPointer<Document> doc = document("\n"
+                                            "enum {\n"
+                                            "E1,\n"
+                                            "E2=10,\n"
+                                            "E3\n"
+                                            "};\n"
+                                            );
+
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+    Enum *e = doc->globals->memberAt(0)->asEnum();
+    QVERIFY(e);
+    QCOMPARE(e->memberCount(), 3);
+
+    testEnumaratorDeclarator(e, 0, "0");
+    testEnumaratorDeclarator(e, 1, "10");
+    testEnumaratorDeclarator(e, 2, "11");
+}
+
+void tst_Semantic::enum_constantValue4()
+{
+    QSharedPointer<Document> doc = document("\n"
+                                            "enum {\n"
+                                            "E1,\n"
+                                            "E2=E1+10,\n"
+                                            "E3,\n"
+                                            "E4=10,\n"
+                                            "E5\n"
+                                            "};\n"
+                                            );
+
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+    Enum *e = doc->globals->memberAt(0)->asEnum();
+    QVERIFY(e);
+    QCOMPARE(e->memberCount(), 5);
+
+    testEnumaratorDeclarator(e, 0, "0");
+    testEnumaratorDeclarator(e, 1, "E1+10");
+    testEnumaratorDeclarator(e, 2, NULL);
+    testEnumaratorDeclarator(e, 3, "10");
+    testEnumaratorDeclarator(e, 4, "11");
+}
+
+void tst_Semantic::enum_constantValue5()
+{
+    QSharedPointer<Document> doc = document("\n"
+                                            "enum {\n"
+                                            "E1,\n"
+                                            "E2=E1,\n"
+                                            "E3,\n"
+                                            "E4=E3,\n"
+                                            "E5\n"
+                                            "};\n"
+                                            );
+
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+    Enum *e = doc->globals->memberAt(0)->asEnum();
+    QVERIFY(e);
+    QCOMPARE(e->memberCount(), 5);
+
+    testEnumaratorDeclarator(e, 0, "0");
+    testEnumaratorDeclarator(e, 1, "0");
+    testEnumaratorDeclarator(e, 2, "1");
+    testEnumaratorDeclarator(e, 3, "1");
+    testEnumaratorDeclarator(e, 4, "2");
+}
+
+void tst_Semantic::enum_constantValueNegative()
+{
+    QSharedPointer<Document> doc = document(
+                "enum {\n"
+                "  E1=-2,\n"
+                "  E2,\n"
+                "  E3,\n"
+                "  E4\n"
+                "};\n"
+    );
+
+    QCOMPARE(doc->errorCount, 0U);
+    QCOMPARE(doc->globals->memberCount(), 1);
+    Enum *e = doc->globals->memberAt(0)->asEnum();
+    QVERIFY(e);
+    QCOMPARE(e->memberCount(), 4);
+
+    testEnumaratorDeclarator(e, 0, "-2");
+    testEnumaratorDeclarator(e, 1, "-1");
+    testEnumaratorDeclarator(e, 2, "0");
+    testEnumaratorDeclarator(e, 3, "1");
 }
 
 QTEST_MAIN(tst_Semantic)
